/* ----------------------------------------------------------------
 *   FILE
 *	inh.c
 *	
 *   DESCRIPTION
 *	node inheritance graph file - generated by the inherits.sh
 *	script from the contents of the nodes files.
 *
 *   NOTES
 *	NodeIsType() now uses a lookup table instead of doing a
 *	tree walk.  The first time you ask for a node's type, we
 *	walk the tree and initialize the corresponding entries in
 *	the table.  Subsequent calls go directly to the table.
 *
 *   IDENTIFICATION
 *	$Header: /data/01/postgres/src/backend/nodes/RCS/inherits.sh,v 1.13 1993/04/21 23:24:27 marc Exp $
 * ----------------------------------------------------------------
 */

#include "tmp/c.h"
#include "nodes/pg_lisp.h"
#include "nodes/nodes.h"
#include "nodes/primnodes.h"
#include "nodes/execnodes.h"
#include "nodes/relation.h"
#include "nodes/plannodes.h"
#include "nodes/mnodes.h"
#include "tags.h"

struct nodeinfo {
	char	*ni_name;
	TypeId	ni_id;
	TypeId	ni_parent;
	Size	ni_size;
};
struct nodeinfo _NodeInfo[] = {
	{ "Node", T_Node, T_Node, sizeof(struct _Node) },
	{ "Plan", T_Plan, T_Node, sizeof(struct _Plan) },
	{ "Fragment", T_Fragment, T_Node, sizeof(struct _Fragment) },
	{ "Existential", T_Existential, T_Plan, sizeof(struct _Existential) },
	{ "Result", T_Result, T_Plan, sizeof(struct _Result) },
	{ "Append", T_Append, T_Plan, sizeof(struct _Append) },
	{ "Scan", T_Scan, T_Plan, sizeof(struct _Scan) },
	{ "SeqScan", T_SeqScan, T_Scan, sizeof(struct _SeqScan) },
	{ "ScanTemps", T_ScanTemps, T_Plan, sizeof(struct _ScanTemps) },
	{ "IndexScan", T_IndexScan, T_Scan, sizeof(struct _IndexScan) },
	{ "JoinRuleInfo", T_JoinRuleInfo, T_Node, sizeof(struct _JoinRuleInfo) },
	{ "Join", T_Join, T_Plan, sizeof(struct _Join) },
	{ "NestLoop", T_NestLoop, T_Join, sizeof(struct _NestLoop) },
	{ "MergeJoin", T_MergeJoin, T_Join, sizeof(struct _MergeJoin) },
	{ "HashJoin", T_HashJoin, T_Join, sizeof(struct _HashJoin) },
	{ "Temp", T_Temp, T_Plan, sizeof(struct _Temp) },
	{ "Material", T_Material, T_Temp, sizeof(struct _Material) },
	{ "Sort", T_Sort, T_Temp, sizeof(struct _Sort) },
	{ "Agg", T_Agg, T_Temp, sizeof(struct _Agg) },
	{ "Unique", T_Unique, T_Temp, sizeof(struct _Unique) },
	{ "Hash", T_Hash, T_Plan, sizeof(struct _Hash) },
	{ "Choose", T_Choose, T_Plan, sizeof(struct _Choose) },
	{ "Resdom", T_Resdom, T_Node, sizeof(struct _Resdom) },
	{ "Fjoin", T_Fjoin, T_Node, sizeof(struct _Fjoin) },
	{ "Expr", T_Expr, T_Node, sizeof(struct _Expr) },
	{ "Var", T_Var, T_Expr, sizeof(struct _Var) },
	{ "Oper", T_Oper, T_Expr, sizeof(struct _Oper) },
	{ "Const", T_Const, T_Expr, sizeof(struct _Const) },
	{ "Param", T_Param, T_Expr, sizeof(struct _Param) },
	{ "Func", T_Func, T_Expr, sizeof(struct _Func) },
	{ "Array", T_Array, T_Expr, sizeof(struct _Array) },
	{ "ArrayRef", T_ArrayRef, T_Expr, sizeof(struct _ArrayRef) },
	{ "Rel", T_Rel, T_Node, sizeof(struct _Rel) },
	{ "SortKey", T_SortKey, T_Node, sizeof(struct _SortKey) },
	{ "Path", T_Path, T_Node, sizeof(struct _Path) },
	{ "IndexPath", T_IndexPath, T_Path, sizeof(struct _IndexPath) },
	{ "JoinPath", T_JoinPath, T_Path, sizeof(struct _JoinPath) },
	{ "MergePath", T_MergePath, T_JoinPath, sizeof(struct _MergePath) },
	{ "HashPath", T_HashPath, T_JoinPath, sizeof(struct _HashPath) },
	{ "OrderKey", T_OrderKey, T_Node, sizeof(struct _OrderKey) },
	{ "JoinKey", T_JoinKey, T_Node, sizeof(struct _JoinKey) },
	{ "MergeOrder", T_MergeOrder, T_Node, sizeof(struct _MergeOrder) },
	{ "CInfo", T_CInfo, T_Node, sizeof(struct _CInfo) },
	{ "JoinMethod", T_JoinMethod, T_Node, sizeof(struct _JoinMethod) },
	{ "HInfo", T_HInfo, T_JoinMethod, sizeof(struct _HInfo) },
	{ "MInfo", T_MInfo, T_JoinMethod, sizeof(struct _MInfo) },
	{ "JInfo", T_JInfo, T_Node, sizeof(struct _JInfo) },
	{ "Iter", T_Iter, T_Node, sizeof(struct _Iter) },
	{ "Stream", T_Stream, T_Node, sizeof(struct _Stream) },
	{ "IndexInfo", T_IndexInfo, T_Node, sizeof(struct _IndexInfo) },
	{ "RelationInfo", T_RelationInfo, T_Node, sizeof(struct _RelationInfo) },
	{ "TupleCount", T_TupleCount, T_Node, sizeof(struct _TupleCount) },
	{ "TupleTableSlot", T_TupleTableSlot, T_LispValue, sizeof(struct _TupleTableSlot) },
	{ "ExprContext", T_ExprContext, T_Node, sizeof(struct _ExprContext) },
	{ "ProjectionInfo", T_ProjectionInfo, T_Node, sizeof(struct _ProjectionInfo) },
	{ "JunkFilter", T_JunkFilter, T_JunkFilter, sizeof(struct _JunkFilter) },
	{ "EState", T_EState, T_Node, sizeof(struct _EState) },
	{ "HookNode", T_HookNode, T_Node, sizeof(struct _HookNode) },
	{ "BaseNode", T_BaseNode, T_Node, sizeof(struct _BaseNode) },
	{ "CommonState", T_CommonState, T_BaseNode, sizeof(struct _CommonState) },
	{ "ResultState", T_ResultState, T_CommonState, sizeof(struct _ResultState) },
	{ "AppendState", T_AppendState, T_BaseNode, sizeof(struct _AppendState) },
	{ "CommonScanState", T_CommonScanState, T_CommonState, sizeof(struct _CommonScanState) },
	{ "ScanState", T_ScanState, T_CommonScanState, sizeof(struct _ScanState) },
	{ "ScanTempState", T_ScanTempState, T_CommonScanState, sizeof(struct _ScanTempState) },
	{ "IndexScanState", T_IndexScanState, T_CommonState, sizeof(struct _IndexScanState) },
	{ "JoinState", T_JoinState, T_CommonState, sizeof(struct _JoinState) },
	{ "NestLoopState", T_NestLoopState, T_JoinState, sizeof(struct _NestLoopState) },
	{ "MergeJoinState", T_MergeJoinState, T_JoinState, sizeof(struct _MergeJoinState) },
	{ "HashJoinState", T_HashJoinState, T_JoinState, sizeof(struct _HashJoinState) },
	{ "MaterialState", T_MaterialState, T_CommonScanState, sizeof(struct _MaterialState) },
	{ "AggState", T_AggState, T_CommonScanState, sizeof(struct _AggState) },
	{ "SortState", T_SortState, T_CommonScanState, sizeof(struct _SortState) },
	{ "UniqueState", T_UniqueState, T_CommonState, sizeof(struct _UniqueState) },
	{ "HashState", T_HashState, T_CommonState, sizeof(struct _HashState) },
	{ "MemoryContext", T_MemoryContext, T_Node, sizeof(struct _MemoryContext) },
	{ "GlobalMemory", T_GlobalMemory, T_MemoryContext, sizeof(struct _GlobalMemory) },
	{ "PortalMemoryContext", T_PortalMemoryContext, T_MemoryContext, sizeof(struct _PortalMemoryContext) },
	{ "PortalVariableMemory", T_PortalVariableMemory, T_PortalMemoryContext, sizeof(struct _PortalVariableMemory) },
	{ "PortalHeapMemory", T_PortalHeapMemory, T_PortalMemoryContext, sizeof(struct _PortalHeapMemory) },
	{ "LispValue", T_LispValue, T_Node, sizeof(struct _LispValue) },
	{ "LispSymbol", T_LispSymbol, T_Node, sizeof(struct _LispSymbol) },
	{ "LispList", T_LispList, T_Node, sizeof(struct _LispList) },
	{ "LispInt", T_LispInt, T_Node, sizeof(struct _LispInt) },
	{ "LispFloat", T_LispFloat, T_Node, sizeof(struct _LispFloat) },
	{ "LispVector", T_LispVector, T_Node, sizeof(struct _LispVector) },
	{ "LispStr", T_LispStr, T_Node, sizeof(struct _LispStr) },
	{ "INVALID", 0, 0, 0 }
};

#define _NClasses 	(lengthof(_NodeInfo) - 1)
#define _NClassBytes	((_NClasses / 8) + 1)

TypeId _InvalidTypeId = (TypeId) _NClasses;
bits8  _NodeClassArray[_NClasses][_NClassBytes];    
bits8  _NodeBitMask[] = { 1<<0, 1<<1, 1<<2, 1<<3, 1<<4, 1<<5, 1<<6, 1<<7 };

bool   _NodeInfoTrace = false;

/* ----------------
 *	InitNodeArray initializes our node's inheritance table.
 * ----------------
 */
void
InitNodeArray(i)
    register TypeId i;
{
    register TypeId j;
    register int    q;
    register int    r;

    /* ----------------
     *	base clause for recursion: initializing the root node
     * ----------------
     */
    if (i == T_Node) {
	_NodeClassArray[i][0] = 1<<0;
	return;
    }

    /* ----------------
     *	recursive initialization: initialize our parent, if necessary
     * ----------------
     */
    j = _NodeInfo[i].ni_parent;
    if (_NodeClassArray[j][0] == 0)
	InitNodeArray(j);

    /* ----------------
     *	now initialize ourself by copying our parent's bitmask and
     *  setting our own bit.
     * ----------------
     */
    for (q=0; q<_NClassBytes; q++)
	_NodeClassArray[i][q] = _NodeClassArray[j][q];
    
    q = i / 8; r = i % 8;
    _NodeClassArray[i][q] |= _NodeBitMask[r];
}

/* ----------------
 *	NodeIsType
 *
 * 	determine if (thisNode) is of type (tag) or is a subclass
 *	of type (tag).  We do this by consulting the master
 *	node class array bitmap: look in the row corresponding
 *	to (thisNode)'s type.  Now look at the bit in the position
 *	indicated by (tag).  If this is set, then (thisNode) is
 *	of type (tag), otherwise it's not.
 *		     
 *	The table is initialized on demand.  If nobody ever asks
 *	about a given class, then we never initialize that class's
 *	information...
 * ----------------
 */
bool
NodeIsType(thisNode, tag)
    Node		thisNode;
    register TypeId 	tag;
{
    register TypeId i;
    register int    q;
    register int    r;
	
    Assert(NodeIsValid(thisNode));
    
    i = NodeType(thisNode);
    Assert(TypeIdIsValid(i));
    Assert(TypeIdIsValid(tag));

    if (_NodeClassArray[i][0] == 0)
	InitNodeArray(i);

    q = tag / 8; r = tag % 8;
    return (bool)
	(_NodeClassArray[i][q] & _NodeBitMask[r]);
}
    
void
Dump_NodeInfo()
{
    register TypeId	i;

    printf("%16.16s%16.16s%16.16s\n", 
	   "NODE NAME:", "NODE TAG:", "PARENT NODE:");

    for (i = 0; i < _InvalidTypeId; ++i)
	printf("%16.16s%16.1d%16.16s\n",
	       _NodeInfo[i].ni_name,
	       _NodeInfo[i].ni_id, 
	       _NodeInfo[_NodeInfo[i].ni_parent].ni_name);
}

