/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_open_channel(), gdi_close_channel(), gdi_channel_is_open(),
 *	gdi_flush()
 *
 * DESCRIPTION
 *	This file contains functions for managing database channels.
 *
 * DIAGNOSTICS
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *	gdi_channel.c (this file)
 *
 * AUTHOR
 *	B. MacRitchie
 */


#include <stddef.h>
#include <stdlib.h>
#include "utilities.h"
#include "libgendb.h"


#ifndef	lint
static char SccsId[] = "@(#)gdi_channel.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif

/*
 * gdi_open_channel ()
 *
 * Open a database channel.
 *
 * Public
 */

int
gdi_open_channel (conn, channo)
dbConn	*conn;
int	*channo;
{
     int    ret;
     char   *fname = "gdi_open_channel: ";
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT,
			 "gdi_open_channel: Not connected to database");
	  return (GDI_FAILURE);
     }

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->open_channel) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->open_channel (conn, channo)) != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_open_channel: Vendor function unavailable");

     return (GDI_FAILURE);
}




/*
 * gdi_close_channel ()
 *
 * Close a database channel.
 *
 * Public
 */

int
gdi_close_channel (conn, channo)
dbConn	*conn;
int	channo;
{
     int    ret;
     char   *fname = "gdi_close_channel: ";
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT,
			 "gdi_close_channel: Not connected to database");
	  return (GDI_FAILURE);
     }

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->close_channel) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->close_channel (conn, channo)) != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_close_channel: Vendor function unavailable");

     return (GDI_FAILURE);
}





/*
 * gdi_channel_is_open ()
 *
 * Determine if a specified channel is open.  Return TRUE or FALSE.
 *
 * Public
 */

int
gdi_channel_is_open (conn, channo)
dbConn	*conn;
int	channo;
{
     int    ret;
     char   *fname = "gdi_channel_is_open: ";
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT,
			 "gdi_channel_is_open: Not connected to database");
	  return (GDI_FAILURE);
     }

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->channel_is_open) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->channel_is_open (conn, channo)) != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_channel_is_open: Vendor function unavailable");
     return (FALSE);
}



/*
 * gdi_flush ()
 *
 * Flush a database channel.
 *
 * Public
 */

int
gdi_flush (conn, channo)
dbConn	*conn;
int	channo;
{
     int    ret;
     char   *fname = "gdi_flush: ";
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT,
			 "gdi_flush: Not connected to database");
	  return (GDI_FAILURE);
     }

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->flush) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->flush (conn, channo)) != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_flush: Vendor function unavailable");

     return (GDI_FAILURE);
}








