/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *
 *	gdi_coldef.c
 *
 * SYNOPSIS
 *
 *	#include "libgendb.h"
 *
 *
 * DESCRIPTION
 *
 *	This file contains functions for accessing the dbColDef of the
 *	Generic Database Interface.
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *
 *	gdi_coldef.c (this file)
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgendb.h
 *
 * AUTHOR
 *
 *	B. MacRitchie
 */


#include <stddef.h>
#include <stdlib.h>
#include "utilities.h"
#include "libgendb.h"


#ifndef	lint
static char SccsId[] = "@(#)gdi_coldef.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif



/*
 * gdi_col_def_create ()
 *
 * Create a new Column Def
 *
 * Public
 */

dbColDef *
gdi_col_def_create (name, ctype, allow_null, max_strlen, max_arrlen,
		    dbtype, precision, scale, dbtype_s)
char	*name;
dbCtype ctype;
int	allow_null;
int	max_strlen;
int     max_arrlen;
int	dbtype;
int	precision;
int	scale;
char	*dbtype_s;
{
     dbColDef	*coldef;
     
     if ((coldef = UCALLOC (dbColDef, 1)) == NULL)
	  return (NULL);
     
     strncpy (coldef->name, name, GDI_MAX_NAME);
     coldef -> allow_null = allow_null;
     coldef -> ctype = ctype;
     coldef -> max_strlen = max_strlen;
     coldef -> max_arrlen = max_arrlen;
     coldef -> dbtype = dbtype;
     coldef -> precision = precision;
     coldef -> scale = scale;
     strncpy (coldef->dbtype_s, dbtype_s, GDI_MAX_TYPE);

     return (coldef);
}



/*
 * gdi_col_def_destroy ()
 *
 * Destroy a Column Def
 *
 * Public
 */

int
gdi_col_def_destroy (coldef)
dbColDef	*coldef;
{
     UFREE (coldef);
     return (GDI_SUCCESS);
}



/*
 * gdi_col_def_add ()
 *
 * Add a Column Def to a dbObj
 *
 * Public
 */

int
gdi_obj_col_def_add (obj, coldef)
dbObj		*obj;
dbColDef	*coldef;
{
     int	n;
     dbColDef	**c;
     
     if (obj == NULL)
	  return (GDI_FAILURE);
     
     n = GDI_OBJ_NUM_COLUMNS (obj);
     
     if ((c = UREALLOC (obj->col_def, dbColDef *, n + 2)) 
	 == NULL)
	  return (GDI_FAILURE);

     obj->col_def = c;

     obj->col_def [n] = coldef;
     obj->col_def [n + 1] = NULL;

     GDI_OBJ_NUM_COLUMNS (obj) ++;
     
     return (GDI_SUCCESS);
}
