/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_create.c
 *
 * SYNOPSIS
 *
 *	#include "libgendb.h"
 *
 * DESCRIPTION
 *
 *	This file contains functions for the database Create Tables
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *
 *	gdi_create.c (this file)
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgendb.h
 *
 * AUTHOR
 *	B. MacRitchie
 */


#include <stddef.h>
#include <stdlib.h>
#include "utilities.h"
#include "libgendb.h"

#ifndef	lint
static char SccsId[] = "@(#)gdi_create.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif


/*
 * gdi_create_table ()
 *
 * Create a database table
 *
 * Public
 */

int
gdi_create_table (conn, table_name, table_def)
dbConn		*conn;
char		*table_name;
dbObj		*table_def;
{
     int    ret;
     char   *fname = "gdi_abort: ";
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT,
			 "gdi_abort: Not connected to database");
	  return (GDI_FAILURE);
     }
     
     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->create_table) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->create_table (conn, table_name, table_def))
	      != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_create_table: Vendor function unavailable");

     return (GDI_FAILURE);
}



