/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_default.c
 *
 * SYNOPSIS
 *
 *	#include "libgendb.h"
 *
 *
 * DESCRIPTION
 *
 *	This file contains functions for creating the default dbTuple of the
 *	Generic Database Interface.
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *
 *	gdi_default.c (this file)
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgendb.h
 *
 * AUTHOR
 */


#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include "utilities.h"
#include "libgendb.h"


#ifndef	lint
static char SccsId[] = "@(#)gdi_default.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif


/*
 * macros for Access of default dbTuple
 */

#define GDI_DEF_TUPLE_NUM_COLUMNS(tuple)	((tuple)->n_columns)
#define GDI_DEF_TUPLE_NAME(tuple, col_num)	((tuple)->col[col_num].name)
#define GDI_DEF_TUPLE_DBTYPE_S(tuple, col_num)	((tuple)->col[col_num].dbtype_s)
#define GDI_DEF_TUPLE_CTYPE(tuple, col_num)	((tuple)->col[col_num].ctype)
#define GDI_DEF_TUPLE_LENGTH(tuple, col_num)	((tuple)->col[col_num].str_length)
#define GDI_DEF_TUPLE_STR_LENGTH(tuple, col_num) ((tuple)->col[col_num].str_length)
#define GDI_DEF_TUPLE_ARR_LENGTH(tuple, col_num) ((tuple)->col[col_num].arr_length)
#define GDI_DEF_TUPLE_IS_NULL(tuple, col_num)	((tuple)->col[col_num].is_null)
#define GDI_DEF_TUPLE_VALUE(tuple, col_num)	(&((tuple)->col[col_num].value))

/*
 * prototypes for Constructor Default Tuple
 */

Proto (static int, gdi_def_tuple_add, 
       (
	dbTuple  ***tuple_array, 	/* tuple array pointer */
	dbTuple  *tuple,    		/* tuple               */
	void     **args                 /* pointer to constructor arguments */
	));

Proto (static dbTuple *, gdi_def_tuple_retrieve, 
       (
	dbTuple  **tuple, 		/* tuple pointer     */
	int      tuple_num,		/* tuple number      */
	void     **args                 /* pointer to constructor arguments */
	));

Proto (static int, gdi_def_tuple_destroy, 
       (
	dbTuple  *tuple,		/* tuple             */
	void     **args                 /* pointer to constructor arguments */
	));

Proto (static dbTuple *, gdi_def_tuple_create, 
       (
	dbColDef **col_def, 		/* column defintion  */
	void     **args                 /* pointer to constructor arguments */
	));

Proto (static int, gdi_def_tuple_fill_value, 
       (
	int      col_num, 		/* column number     */
	dbColDef *col_def, 		/* column definition */
	dbTuple  *tuple, 		/* tuple             */
	void     *dbdata, 		/* database data     */
	int	 is_null, 		/* "is null" flag    */
	int      str_length,		/* string length (including NULL terminator) */
	int      arr_length,		/* array length      */
	void     **args                 /* pointer to constructor arguments          */
	));

Proto (static void *, gdi_def_tuple_get_value, 
       (
	int      col_num, 		/* column number     */
	dbColDef *col_def, 		/* column definition */
	dbTuple  *tuple, 		/* tuple             */
	int      *is_null, 		/* "is null" flag    */
	int      *str_length,		/* string length (including NULL terminator) */
	int      *arr_length,		/* array length      */
	void     **args                 /* pointer to constructor arguments          */
	));


#ifdef __STDC__
dbConstr	GDI_DEFAULT = 
{
     (int    (*) ())	                        NULL,
     (void * (*) ())			        NULL,
     (int    (*) (void **, void *, void **))	gdi_def_tuple_add,
     (void * (*) (void *, int, void **))	gdi_def_tuple_retrieve,
     (int    (*) (void *, void **))		gdi_def_tuple_destroy,
     (void * (*) (dbColDef **, void **))        gdi_def_tuple_create,
     (int    (*) (int, dbColDef *, void *, void *, int, int, int, void **))	
	                                        gdi_def_tuple_fill_value,
     (void * (*) (int, dbColDef *, void *, int *, int *, int *, void **)) 
	                                        gdi_def_tuple_get_value,
     (int    (*) ())                            NULL,
     (void *)	NULL,
     (dbCoerce)	GDI_DEFAULT_COERCE,
};
#else
dbConstr	GDI_DEFAULT = 
{
     (int    (*) ())	NULL,
     (void * (*) ())	NULL,
     (int    (*) ())	gdi_def_tuple_add,
     (void * (*) ())	gdi_def_tuple_retrieve,
     (int    (*) ())	gdi_def_tuple_destroy,
     (void * (*) ())	gdi_def_tuple_create,
     (int    (*) ())	gdi_def_tuple_fill_value,
     (void * (*) ()) 	gdi_def_tuple_get_value,
     (int    (*) ())	NULL,
     (void *)	NULL,
     (dbCoerce)	GDI_DEFAULT_COERCE,
};
#endif



/*
 * gdi_def_tuple_add ()
 *
 * Add a tuple to the tuple array.
 *
 * Private
 */
static
int
gdi_def_tuple_add (tarray, tuple, args)
dbTuple		***tarray;
dbTuple		*tuple;
void            **args;
{
     int	i;
     dbTuple	**t;
     char	*routine = "gdi_def_tuple_add: ";
     
     if (tarray == NULL)
	  return (GDI_FAILURE);
     
     for (i = 0; (*tarray != NULL) && (*tarray)[i] != NULL; i++);

     if ((t = UREALLOC ((*tarray), dbTuple *, i + 2)) == NULL)
     {
	  gdi_error_unix (NULL, routine);
	  return (GDI_FAILURE);
     }
     
     (*tarray) = t;
     
     (*tarray) [i] = tuple;
     (*tarray) [i+1] = NULL;

     return (GDI_SUCCESS);
}


/*
 * gdi_def_tuple_retrieve ()
 *
 * Retrieve a tuple from the tuple array.
 *
 * Private
 */
static
dbTuple *
gdi_def_tuple_retrieve (tarray, tuple_num, args)
dbTuple		**tarray;
int		tuple_num;
void            **args;
{
     if (tarray == NULL)
	  return (NULL);
     
     return (tarray [tuple_num]);
}


/*
 * gdi_def_tuple_destroy ()
 *
 * Destroy a tuple.
 *
 * Private
 */
static
int
gdi_def_tuple_destroy (tuple, args)
dbTuple		*tuple;
void            **args;
{
     int	n;
     
     if (tuple == NULL)
	  return (GDI_SUCCESS);
     
     for (n = 0; n < GDI_DEF_TUPLE_NUM_COLUMNS (tuple); n++)
	  if (GDI_DEF_TUPLE_CTYPE (tuple, n) == GDI_STRING)
	       UFREE (GDI_DEF_TUPLE_VALUE (tuple, n)->string);

     UFREE (tuple->col);
     UFREE (tuple);

     return (GDI_SUCCESS);
}     
	  
     
/*
 * gdi_def_tuple_create ()
 *
 * Create a new tuple.
 *
 * Private
 */
static
dbTuple *
gdi_def_tuple_create (coldef, args)
dbColDef	**coldef;
void		**args;
{
     int	i;
     int	n;
     dbTuple	*tuple;
     char	*routine = "gdi_def_tuple_create: ";
     
     if (coldef == NULL)
	  return (NULL);
     
     if ((tuple = UCALLOC (dbTuple, 1)) == NULL)
     {
	  gdi_error_unix (NULL, routine);
	  return (NULL);
     }
     
     for (n = 0; coldef[n] != NULL; n++);
     if ((tuple->col = UCALLOC (dbCol, n)) == (dbCol *) NULL)
     {
	  gdi_error_unix (NULL, routine);
	  UFREE (tuple);
	  return (NULL);
     }
     
     GDI_DEF_TUPLE_NUM_COLUMNS(tuple) = n;

     for (i = 0; i < n; i++)
     {
	  strncpy (GDI_DEF_TUPLE_NAME (tuple, i), coldef[i]->name, GDI_MAX_NAME);
	  strncpy (GDI_DEF_TUPLE_DBTYPE_S (tuple, i), coldef[i]->dbtype_s, GDI_MAX_TYPE);
	  GDI_DEF_TUPLE_CTYPE (tuple, i) = coldef[i]->ctype;
     }
     return (tuple);
}


/*
 * gdi_def_tuple_fill_value ()
 *
 * Fill a tuple column value with database data.  Length for a string
 * may or may not include a NULL terminator.
 *
 * Private
 */
static
int
gdi_def_tuple_fill_value (col_num, col_def, tuple, dbdata, is_null, 
			  str_length, arr_length, args)
int		col_num;
dbColDef	*col_def;
dbTuple		*tuple;
void		*dbdata;
int		is_null;
int		str_length;
int		arr_length;
void            **args;
{
     char	*routine = "gdi_def_tuple_fill_value: ";
     
     if (col_num < 0)
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_def_tuple_fill_value: invalid column number");
	  return (GDI_FAILURE);
     }
     
     if ((col_def == NULL) || (tuple == NULL) || (dbdata == NULL))
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_def_tuple_fill_value: NULL pointer argument");
	  return (GDI_FAILURE);
     }
     
     GDI_DEF_TUPLE_IS_NULL (tuple, col_num) = is_null;
     GDI_DEF_TUPLE_STR_LENGTH (tuple, col_num) = str_length;
     GDI_DEF_TUPLE_ARR_LENGTH (tuple, col_num) = arr_length;

     switch (GDI_DEF_TUPLE_CTYPE (tuple, col_num))
     {
     case GDI_CHAR:
	  GDI_DEF_TUPLE_VALUE (tuple, col_num)->char_val = * ((char *) dbdata);
	  break;
     case GDI_INTEGER:
	  GDI_DEF_TUPLE_VALUE (tuple, col_num)->long_val = * ((int *) dbdata);
	  break;
     case GDI_LONG:
	  GDI_DEF_TUPLE_VALUE (tuple, col_num)->long_val = * ((long *) dbdata);
	  break;
     case GDI_DOUBLE:
	  GDI_DEF_TUPLE_VALUE (tuple, col_num)->double_val = * ((double *) dbdata);
	  break;
     case GDI_FLOAT:
	  GDI_DEF_TUPLE_VALUE (tuple, col_num)->double_val = * ((float *) dbdata);
	  break;
     case GDI_STRING:
	  if ((GDI_DEF_TUPLE_VALUE (tuple, col_num)->string 
	       = UCALLOC (char, str_length + 1)) == NULL)
	  {
	       gdi_error_unix (NULL, routine);
	       return (GDI_FAILURE);
	  }
	  /* 
	   * use bcopy instead of strncpy in case string contains
	   * embedded NULLs.
	   */
	  bcopy ((char *) dbdata , GDI_DEF_TUPLE_VALUE (tuple, col_num)->string, 
		 str_length * sizeof (char));
	  GDI_DEF_TUPLE_VALUE (tuple, col_num)->string [str_length] = '\0';
	  break;
     default:
	  break;
     }
     return (GDI_SUCCESS);
}


/*
 * gdi_def_tuple_get_value ()
 *
 * Return a tuple column value.  Also fill in is_null and length.
 *
 * Private
 */
static
void *
gdi_def_tuple_get_value (col_num, col_def, tuple, is_null, str_length, arr_length, args)
int		col_num;
dbColDef	*col_def;
dbTuple		*tuple;
int		*is_null;
int		*str_length;
int		*arr_length;
void            **args;
{
     void	*dbdata;
     
     if (col_num < 0)
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_def_tuple_get_value: invalid column number");
	  return (NULL);
     }
     
     if (tuple == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_def_tuple_get_value: NULL pointer argument");
	  return (NULL);
     }
     
     if (is_null != NULL)
	  *is_null = GDI_DEF_TUPLE_IS_NULL (tuple, col_num);
     
     if (str_length != NULL)
	  *str_length = GDI_DEF_TUPLE_STR_LENGTH (tuple, col_num);
     
     if (arr_length != NULL)
	  *arr_length = GDI_DEF_TUPLE_ARR_LENGTH (tuple, col_num);
     
     if (col_def->ctype == GDI_STRING)
	  dbdata = (void *) GDI_DEF_TUPLE_VALUE (tuple, col_num)->string;
     else
	  dbdata = (void *) GDI_DEF_TUPLE_VALUE (tuple, col_num);
     
     return (dbdata);
}

