/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	f77_constructor.c
 *
 * SYNOPSIS
 *
 *     Contains FORTRAN constructor functions.
 *
 * DESCRIPTION
 *
 *
 * DIAGNOSTICS
 *
 *
 * FILES
 *
 *	gdi_f77_constr.c (this file)
 *
 * NOTES
 *      Currently only works for reading from postgres.
 *
 * SEE ALSO
 *
 *	libgdi.h
 *
 * AUTHOR
 *	H. Turner
 */


#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include "utilities.h"
#include "libgdi.h"
#include "f77defs.h"

#ifndef	lint
static char SccsId[] = "@(#)gdi_f77_constr.c	16.4 8/24/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif


/* extern dbConn F77_conn;
 * static dbConn *dbconn = &F77_conn;
 */


Proto (static int, f77_def_tuple_add, 
       (
	dbTuple  ***tuple_array, 	/* tuple array pointer */
	dbTuple  *tuple,    		/* tuple               */
	void     **args                 /* pointer to constructor arguments */
	));

Proto (static dbTuple *, f77_def_tuple_retrieve, 
       (
	dbTuple  **tuple, 		/* tuple pointer     */
	int      tuple_num,		/* tuple number      */
	void     **args                 /* pointer to constructor arguments */
	));

Proto (static int, f77_def_tuple_destroy, 
       (
	dbTuple  *tuple,		/* tuple             */
	void     **args                 /* pointer to constructor arguments */
	));

Proto (static dbTuple *, f77_def_tuple_create, 
       (
	dbColDef **col_def, 		/* column defintion  */
	void     **args                 /* pointer to constructor arguments */
	));

Proto (static int, f77_def_fill_data, 
       (
	int      col_num, 		/* column number     */
	dbColDef *col_def, 		/* column definition */
	dbTuple  *tuple, 		/* tuple             */
	void     *dbdata, 		/* database data     */
	int	 is_null, 		/* "is null" flag    */
	int      str_length,		/* string length (including NULL terminator) */
	int      arr_length,		/* array length      */
	void     **args                 /* pointer to constructor arguments          */
	));

Proto (static void *, f77_def_get_data, 
       (
	int      col_num, 		/* column number     */
	dbColDef *col_def, 		/* column definition */
	dbTuple  *tuple, 		/* tuple             */
	int      *is_null, 		/* "is null" flag    */
	int      *str_length,		/* string length (including NULL terminator) */
	int      *arr_length,		/* array length      */
	void     **args                 /* pointer to constructor arguments          */
	));


     

/* default FORTRAN contructor */
#ifdef __STDC__

dbConstr F77_DEF_CONSTR = 
{
     (int    (*) ())	                        NULL,
     (void * (*) ())			        NULL,
     (int    (*) (void **, void *, void **))	f77_def_tuple_add,
     (void * (*) (void *, int, void **))	f77_def_tuple_retrieve,
     (int    (*) (void *, void **))		f77_def_tuple_destroy,
     (void * (*) (dbColDef **, void **))        f77_def_tuple_create,
     (int    (*) (int, dbColDef *, void *, void *, int, int, int, void **))	
	                                        f77_def_fill_data,
     (void * (*) (int, dbColDef *, void *, int *, int *, int *, void **)) 
	                                        f77_def_get_data,
     (int    (*) ())                            NULL,
     (void *)	NULL,
     (dbCoerce)	GDI_DBL_STR_COERCE,
};
#else

dbConstr F77_DEF_CONSTR = 
{
     (int    (*) ())	NULL,
     (void * (*) ())	NULL,
     (int    (*) ())	f77_def_tuple_add,
     (void * (*) ())	f77_def_tuple_retrieve,
     (int    (*) ())	f77_def_tuple_destroy,
     (void * (*) ())	f77_def_tuple_create,
     (int    (*) ())	f77_def_fill_data,
     (void * (*) ()) 	f77_def_get_data,
     (int    (*) ())	NULL,
     (void *)	NULL,
     (dbCoerce)	GDI_DBL_STR_COERCE,
};
#endif


static dbTuple dummy_tuple;

static 
void
fill_dummy_tuple()
{
	dummy_tuple.col = (dbCol *)NULL;
	dummy_tuple.n_columns = 0;
}

/* find the column name in the f77 mapping references */
static 
Col_Name_Addr *find_column_by_name(f77_col_def, name)
Col_Name_Addr *f77_col_def;
char *name;

{
	if ((f77_col_def->column_name == (char *)NULL) ||
            (strlen(f77_col_def->column_name) == 0))
	{
		/* bad f77 mapping info. */
		return ((Col_Name_Addr *)NULL);
	}

	while (f77_col_def != (Col_Name_Addr *)NULL)
	{
		if (STREQL(f77_col_def->column_name,name))
		{
			return (f77_col_def);
		}
		f77_col_def = f77_col_def->next;
	}
	return ((Col_Name_Addr *)NULL);
}

/*
 * routines for the FORTRAN default constructor 
 */



static
int
f77_def_tuple_add (tarray, tuple, args)
dbTuple		***tarray;
dbTuple		*tuple;
void            **args;
{
	fill_dummy_tuple();
	return (GDI_SUCCESS);	
}


static
dbTuple *
f77_def_tuple_retrieve (tarray, tuple_num, args)
dbTuple		**tarray;
int		tuple_num;
void            **args;
{
	fill_dummy_tuple();
	return (&dummy_tuple);

}

static
int
f77_def_tuple_destroy (tuple, args)
dbTuple		*tuple;
void            **args;
{
	return (GDI_SUCCESS);
}

static
dbTuple *
f77_def_tuple_create (coldef, args)
dbColDef	**coldef;
void		**args;
{
	fill_dummy_tuple();
	return (&dummy_tuple);
}


/* get routine for constructor */

static
void 
*f77_def_get_data( col_num, col_def, tuple, is_null, 
		     str_length, arr_length, args)
int		col_num;
dbColDef	*col_def;
dbTuple		*tuple;
int		*is_null;
int		*str_length;
int		*arr_length;
void            **args;
{


	/* args contains a pointer to a linked list of structures
	 * that describe the data in FORTRAN space.  We will copy
	 * the data dircectly from FORTRAN space to dbdata.
	 */

	void *dbdata;
	static double local_double;
	static char   *local_string = (char *)NULL;
	
	char *fname = "f77_def_get_data: ";

        
	Col_Name_Addr *f77_col_def;
	int i, this_row;

	if (args == (void **)NULL)
	{
		return ((void *)NULL);
	}		
	if ((f77_col_def = find_column_by_name(*(Col_Name_Addr **)args,
					      col_def->name)) == (Col_Name_Addr *)NULL)
	{
		return ((void *)NULL);
	}
	
	/* bump the row counter to indicate current position in output array*/
	this_row = f77_col_def->row_count++;

	dbdata = (void *)&local_double;
	
	switch (f77_col_def->type)
	{
	case GDI_LONG:
	case GDI_INTEGER:
		local_double = (double)(((int *)f77_col_def->addr)[this_row]);
		break;
	case GDI_FLOAT:
		local_double = (double)((float *)f77_col_def->addr)[this_row];
		break;
	case GDI_STRING:
		{
			local_string = UREALLOC(local_string, 
						char, 
						f77_col_def->string_len + 1);

			if (local_string == (char *)NULL)
			{
				dbdata = (double *)NULL;
				*str_length = 0;
			}
			else
			{

				char *to = local_string;
				char *from = (char *)&((char *)f77_col_def->addr)[this_row*f77_col_def->string_len];
				dbdata = (double *)local_string;
				*str_length = f77_col_def->string_len + 1;
				
				(void) gdi_for_to_cstr(from, 
						       *str_length -1,
						       to,
						       *str_length);
			}			
		}
		
		break;
	case GDI_DOUBLE:
		local_double = ((double *)f77_col_def->addr)[this_row];
		break;
	case GDI_SHORT:
		local_double= (double)((short *)f77_col_def->addr)[this_row];
		break;
	case GDI_CHAR:
		local_double = (double)((char *)f77_col_def->addr)[this_row];
		break;
	case GDI_UNDEFINED:
	case GDI_USER_DEFINED:
	default:
		dbdata = (double *)NULL;
		/* error */
		break;
	}
	
	*arr_length = 0;
	
	return (dbdata);
}




/* fill routine for constructor */
static
int f77_def_fill_data( col_num, col_def, tuple, dbdata, is_null, 
		      str_length, arr_length, args)
int      col_num;
dbColDef *col_def;
dbTuple  *tuple;
void     *dbdata;
int      is_null;
int      str_length;
int      arr_length;
void     **args;

{
	/* args contains a pointer to a linked list of structures
	 * that describe the data in FORTRAN space.  We will copy
	 * the data from dbdata directly to FORTRAN space.
	 */

        char *fname = "f77_def_fill_data: ";	
	Col_Name_Addr *f77_col_def;
	int i, this_row, aloop, max_arrlen;
	
	if ((f77_col_def = find_column_by_name(*(Col_Name_Addr **)args,
					      col_def->name)) == (Col_Name_Addr *)NULL)
	{
		char buf[500];
		sprintf (buf, 
			 "F77_def_fill_data__: Could not find column name %s",
			 col_def->name ? col_def->name : "<null>");
		gdi_error_app((dbConn*)NULL,GDI_BADDATA,buf);
		return (GDI_FAILURE);
	}
	

	max_arrlen = MIN(arr_length, f77_col_def->array_size);
	
	/* bump the row counter to indicate current position in input array*/
	this_row = f77_col_def->row_count++;
	
	switch (f77_col_def->type)
	{
	case GDI_LONG:
		if (max_arrlen > 0)
		{
			long *to = (long *)f77_col_def->addr;
			double *from = (double*)dbdata;
			
			for (aloop = 0; aloop < max_arrlen; aloop++)
			{
				(to + this_row*max_arrlen)[aloop] = (long)*from;
				from++;
			}
		}
		else
		{
			((long *)f77_col_def->addr)[this_row] = (long)*(double *)dbdata;
		}
		break;

	case GDI_INTEGER:
		if (max_arrlen > 0)
		{
			int *to = (int *)f77_col_def->addr;
			double *from = (double*)dbdata;
			
			for (aloop = 0; aloop < max_arrlen; aloop++)
			{
				(to + this_row*max_arrlen)[aloop] = (int)*from;
				from++;
			}
		}
		else
		{
			((int *)f77_col_def->addr)[this_row] = (int)*(double *)dbdata;
		}
		break;

	case GDI_FLOAT:
		if (max_arrlen > 0)
		{
			float *to = (float *)f77_col_def->addr;
			double *from = (double*)dbdata;
			for (aloop = 0; aloop < max_arrlen; aloop++)
			{
				(to + this_row*max_arrlen)[aloop] = (float)*from;				
				from++;
			}
		}
		else
		{
			((float *)f77_col_def->addr)[this_row] = (float)*(double *)dbdata;
		}
		
		break;
	case GDI_CHAR:
	case GDI_STRING:
		{
				
			char **from = (char **)dbdata;

			if (max_arrlen > 0)
			{
				char *to = &((char *)f77_col_def->addr)[this_row*f77_col_def->string_len*f77_col_def->array_size
];
				for (aloop = 0; aloop < max_arrlen; aloop++)
				{
					(void) gdi_cstr_to_for(from[aloop], 
							       strlen(from[aloop]),
							       to,
							       f77_col_def->string_len);
                                        to += f77_col_def->string_len;
				}
			}
			else
			{
				char   *to = &((char *)f77_col_def->addr)[this_row*f77_col_def->string_len];
				(void) gdi_cstr_to_for(from, 
						       str_length,
						       to,
						       f77_col_def->string_len);
			}
		}
		break;

	case GDI_DOUBLE:
		if (max_arrlen > 0)
		{
			double *to = (double *)f77_col_def->addr;
			double *from = (double*)dbdata;
			for (aloop = 0; aloop < max_arrlen; aloop++)
			{
				(to + this_row*max_arrlen)[aloop] = (double)*from;				
				from++;
			}
		}
		else
		{

			((double *)f77_col_def->addr)[this_row] = *(double *)dbdata;
		}
		
		break;

	case GDI_SHORT:
		if (max_arrlen > 0)
		{
			short *to = (short *)f77_col_def->addr;
			double *from = (double*)dbdata;
			for (aloop = 0; aloop < max_arrlen; aloop++)
			{
				(to + this_row*max_arrlen)[aloop] = (short)*from;				
				from++;
			}
		}
		else
		{

			((short *)f77_col_def->addr)[this_row] = (short)*(double *)dbdata;
		}
		
		break;

#if 0
	case GDI_CHAR:
		
		((char *)f77_col_def->addr)[this_row] = (char)*(double *)dbdata;
		break;
#endif

	case GDI_UNDEFINED:
	case GDI_USER_DEFINED:
	default:
		/* error */
		{
			char buf[500];
			sprintf (buf, 
				 "F77_def_fill_data__: Undefined or unrecognized data type");
			gdi_error_app((dbConn*)NULL,GDI_BADDATA,buf);
			return (GDI_FAILURE);
		}
		

	}
	return (GDI_SUCCESS);
}
