/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_f77_dbconn.c
 *
 * SYNOPSIS
 *
 *	#include "libgdi.h"
 *
 *
 * DESCRIPTION
 *
 *	This file contains global data structures and its support functions 
 *      for the FORTRAN 77 interface to the gdi.
 *      
 *      FORTRAN will reference its database connections and its mapping
 *      refernces as integers.  Since the gdi is expecting one of its
 *      structures as defined in libgdi.h, we must translate between
 *      what FORTRAN expects and what C expects.
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *
 *	gdi_f77_dbconn.c
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgdi.h
 *
 * AUTHOR
 *	H. Turner
 */


#include "libgdi.h"
#include "utilities.h"
#include "f77defs.h"

#ifndef	lint
static char SccsId[] = "@(#)gdi_f77_dbconn.c	16.2 8/11/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif


#define F77DBINC 10


/* global data strutures */

static int init = 0;


/* dbConn array */
static dbConn **f77_db_conn = (dbConn **)NULL;
static int dbConn_size = 0;



/* perform run-time initializations */
int initialize_db_conn()
{
	int i;
	
	init = 1;/**/
	f77_db_conn = UALLOC( dbConn*, F77DBINC);
	if (f77_db_conn == (dbConn **)NULL)
	{
		return GDI_FAILURE;
	}
	dbConn_size = F77DBINC;
	for (i=0; i<dbConn_size; i++)
	{
		f77_db_conn[i] = (dbConn*)NULL;
	}
	
	return GDI_SUCCESS;
	
	
}

dbConn *get_db_conn(conn)
int conn;
{
	conn--;
	
	if ((conn > dbConn_size) ||
	    (conn < 0))
	{
		return ((dbConn *)NULL);
	}
	return (f77_db_conn[conn]);
}


int add_db_conn(conn)
dbConn *conn;
{
	int i, j;
	
	
	
	if (conn == (dbConn *)NULL)
	{
		return 0;
	}

	for (i=0; i<dbConn_size; i++)
	{
		if (f77_db_conn[i] == (dbConn *)NULL)
		{
			f77_db_conn[i] = conn;
			return (++i);
		}
	}
	
	j = dbConn_size;
	dbConn_size += F77DBINC;
	f77_db_conn = UREALLOC(f77_db_conn, dbConn*, dbConn_size);
	if (f77_db_conn == (dbConn **)NULL)
	{
		/* handle malloc failure */
	}

	for (i=j; i<dbConn_size; i++)
	{
		f77_db_conn[i] = (dbConn*)NULL;
	}
	

	f77_db_conn[j] = conn;
	return (++j);
}

void remove_db_conn(conn)
int conn;

{
	conn--;
	f77_db_conn[conn] = (dbConn *)NULL;
}

