/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_error_init(), 
 *	gdi_error_get(), gdi_error_flags()
 *
 * DESCRIPTION
 *	This file contains the FORTRAN interface to the functions 
 *      for the generic database interface error handler.
 *
 * DIAGNOSTICS
 *	Functions which return pointers return NULL if an error
 *	occurrs.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *	gdi_f77_error.c (this file)
 *	gdi_error.c
 *
 * NOTES
 *	Database-specific errors are set by the appropriate database
 *	vendor specific function.  Database errors are not set at
 *	the GDI level.
 * AUTHOR
 *	H. Turner
 */

#include "utilities.h"
#include "libgdi.h"
#include "f77defs.h"

#ifndef	lint
static char SccsId[] = "@(#)gdi_f77_error.c	16.2 8/11/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif


/*
 * gdi_error_init ()
 *
 * Initializes errors for the connection
 *
 * Public
 */

void
gdi_error_init_ (conn, debug, threshold, reserved1, reserved2)
int		*conn;
int 	        *debug;
int     	*threshold;
int		*reserved1;
int		*reserved2;
{
	(void)gdi_error_init(get_db_conn(*conn),
			     *debug,
			     *threshold,
			     *reserved1,
			     *reserved2);

}



/*
 * gdi_error_get ()
 *
 * Get error settings for the connection.
 *
 * Public
 */

void
gdi_error_get_ (conn, errcode, errtext, maxtext, status, severity, errtext_len)
int		*conn;
int		*errcode;
char		*errtext;
int		*maxtext;
int	        *status;
int	        *severity;
int             errtext_len;

{
	char *temp;
	dbConn *i;
	dbStatus stat;
	dbErrLev sever;
	
	
	if (*conn == gdicommon_.gdi_noconn)
	{
		i = (dbConn *)NULL;
	}
	else
	{
		i = get_db_conn(*conn);
	}

	if ((temp = UALLOC(char, *maxtext+1)) == (char *)NULL)
	{
		gdi_error_unix(i, "gdi_error_get_");
		
	}
	else
	{
		gdi_error_get(i, errcode, temp, *maxtext+1, &stat, &sever);
		*status = (int)stat;
		*severity = (int)sever;
		(void)gdi_cstr_to_for(temp, *maxtext+1, errtext, *maxtext);
	}
	UFREE(temp);	
}



/*
 * gdi_error_flags ()
 *
 * Get error flag settings for the connection.
 *
 * Public
 */

int
gdi_error_flags_ (conn, debug, threshold)
int		*conn;
int		*debug;
int	        *threshold;
{
	dbDebug  dbg;
	dbErrLev thresh;
	int i;
	
	
	i = gdi_error_flags(get_db_conn(*conn),
			    &dbg,
			    &thresh);
	*debug = (int)dbg;
	*threshold = (int)thresh;
	return i;
}
