/* Copyright 1992 Science Applications International Corporation
 * 
 * This software may not be used, copied, modified, or distributed without
 * the express written permission of Science Applications International 
 * Corporation (SAIC).  SAIC makes no warranty of any kind with regard
 * to this software, including, but not limited to, the implied warranties
 * of fitness for a particular purpose.
 * 
 * Thomas C. Bache, Vice President, SAIC, San Diego CA (bache@esosun.css.gov)
 */

/*
 * NAME
 *	gdi_commit()		end a transaction, making changes permanent
 *	gdi_rollback()		end a transaction, discarding changes
 *	gdi_savepoint()		set a savepoint (SYBASE & ORACLE)
 *
 * DESCRIPTION
 *	This file contains FORTRAN interfaces to functions for the 
 *      database transaction management.
 *
 * DIAGNOSTICS
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *	f77_gdi_tran.c (this file)
 *	gdi_tran.c
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgdi.h
 *
 * AUTHOR
 *	H. Turner
 */


#include "libgdi.h"
#include "utilities.h"
#include "f77defs.h"


#ifndef	lint
static char	SccsId[] = "%W% %G%	SAIC";
#endif


/*
 * gdi_commit ()
 *
 * Commit a database transaction.
 *
 * Public
 */

int
gdi_commit_ (conn, channo)
int	*conn;
int	*channo;
{
	return (gdi_commit(get_db_conn(*conn),
			   *channo));
}



/*
 * gdi_rollback ()
 *
 * Rollback a database transaction.
 *
 * Public
 */

int
gdi_rollback_ (conn, channo)
int	*conn;
int	*channo;
{
	return (gdi_rollback(get_db_conn(*conn),
			     channo));
}



/*
 * gdi_savepoint ()
 *
 * Set a "save" point for a database transaction.
 *
 * Public
 */

int
gdi_savepoint_ (conn, channo, sname, sname_len)
int	*conn;
int	*channo;
char	*sname;
int     sname_len;
{

	char *save;
	dbConn *dbconn;
	int i;
	
	dbconn = get_db_conn(*conn);
	
	if ((save = UALLOC(char, sname_len)) == (char *)NULL)
	{
		gdi_error_unix(dbconn, "gdi_savepoint_:");
		UFREE(save);
		return (GDI_FAILURE);
	}
	(void)gdi_for_to_cstr(save, sname_len, sname, sname_len);

	i = gdi_savepoint(dbconn,
			  *channo,
			  save);
	UFREE(save);
	
	return (i);
	
}
