/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_init_
 *
 * SYNOPSIS
 *
 *	#include "libgdi.h"
 *	#include "utilities.h"
 *	#include "f77defs.h"
 *
 *
 * DESCRIPTION
 *
 *      FORTRAN gdi Initialization routine.
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *
 *	gdi_f77_init.c
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgdi.h
 *
 * AUTHOR
 *	H. Turner
 */


#include "libgdi.h"
#include "utilities.h"
#include "f77defs.h"

#ifndef	lint
static char SccsId[] = "@(#)gdi_f77_init.c	16.2 8/11/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif



int gdi_init_(prog_name, len)
char *prog_name;
int len;

{
        int i;
	char *pgname;
        
	int map_status, db_conn_status;
	
	if ((map_status = gdi_initialize_mapping()) != GDI_SUCCESS)
	{
		return (map_status);
	}
	
	if ((db_conn_status = initialize_db_conn()) != GDI_SUCCESS)
	{
		return (db_conn_status);
	}
	

	/* initialize FORTRAN common */

	/* constants */
	gdicommon_.gdi_max_columns     = GDI_MAX_COLUMNS;
	gdicommon_.gdi_max_name        = GDI_MAX_NAME;
	gdicommon_.gdi_max_type        = GDI_MAX_TYPE;
	gdicommon_.gdi_max_keyname     = GDI_MAX_KEYNAME;
	gdicommon_.gdi_error_size      = GDI_ERROR_SIZE;
	gdicommon_.gdi_account_size    = GDI_ACCOUNT_SIZE;
	gdicommon_.gdi_dbname_size     = GDI_DBNAME_SIZE;
	gdicommon_.gdi_tablename_size  = GDI_TABLENAME_SIZE;
	gdicommon_.gdi_node_size       = GDI_NODE_SIZE;

#if 0
	gdicommon_.gdi_max_columns     = 1000;
	gdicommon_.gdi_max_name        = 40;
	gdicommon_.gdi_max_type        = 40;
	gdicommon_.gdi_max_keyname     = 15;
	gdicommon_.gdi_error_size      = 132;
	gdicommon_.gdi_account_size    = 30;
	gdicommon_.gdi_dbname_size     = 30;
	gdicommon_.gdi_tablename_size  = 30;
	gdicommon_.gdi_node_size       = 30;
#endif	

	/* dbDebug  */
	gdicommon_.gdi_debug_off       = (int)GDI_DEBUG_OFF;
	gdicommon_.gdi_debug_on        = (int)GDI_DEBUG_ON;
	gdicommon_.gdi_debug_verbose   = (int)GDI_DEBUG_VERBOSE;

	/* dbStatus   */
	gdicommon_.gdi_success         = (int)GDI_SUCCESS;
	gdicommon_.gdi_failure         = (int)GDI_FAILURE;

	/* dbCtype */
	gdicommon_.gdi_undefined       = (int)GDI_UNDEFINED;
	gdicommon_.gdi_real4           = (int)GDI_FLOAT;	
	gdicommon_.gdi_string          = (int)GDI_STRING;
	gdicommon_.gdi_real8           = (int)GDI_DOUBLE;	
	gdicommon_.gdi_char            = (int)GDI_CHAR;
	gdicommon_.gdi_int4            = (int)GDI_INTEGER;
	gdicommon_.gdi_int2            = (int)GDI_SHORT;


#if 0	
	/*  dbVendor  */
	gdicommon_.gdi_unknown_s       = GDI_UNKNOWN;
	gdicommon_.gdi_sybase_s        = GDI_SYBASE_S;
	gdicommon_.gdi_oracle_s        = GDI_ORACLE_S;
	gdicommon_.gdi_ingres_s        = GDI_INGRES_S;
	gdicommon_.gdi_postgres_s      = GDI_POSTGRESS_S;
#endif
	
	/* dbLang */
	gdicommon_.gdi_sql             = (int)GDI_SQL;
	gdicommon_.gdi_postquel        = (int)GDI_POSTQUEL;
	
	/* dbFormat */
	gdicommon_.gdi_fixed_space     = (int)GDI_FIXED_SPACE;
	gdicommon_.gdi_delimited       = (int)GDI_DELIMITED;

	/* misc.*/
	gdicommon_.gdi_not_used        = (int)GDI_NOT_USED;
	gdicommon_.gdi_no_chan         = (int)GDI_NO_CHAN;
	gdicommon_.gdi_default_chan    = (int)GDI_DEFAULT_CHAN;
	gdicommon_.gdi_select_chan     = (int)GDI_SELECT_CHAN;
	gdicommon_.gdi_update_chan     = (int)GDI_UPDATE_CHAN;
	gdicommon_.gdi_insert_chan     = (int)GDI_INSERT_CHAN;
	gdicommon_.gdi_create_chan     = (int)GDI_CREATE_CHAN; 
	gdicommon_.gdi_drop_chan       = (int)GDI_DROP_CHAN;

	/* dbCorerce */
	gdicommon_.gdi_default_coerce  = (int)GDI_DEFAULT_COERCE;
	gdicommon_.gdi_double_coerce   = (int)GDI_DOUBLE_COERCE;
	gdicommon_.gdi_string_coerce   = (int)GDI_STRING_COERCE;
	gdicommon_.gdi_dbl_str_coerce  = (int)GDI_DBL_STR_COERCE;
	
#if 0
	/* dbQuery */
	gdicommon_.rows_affected       = (int)-1;
	gdicommon_.cmd_num             = (int)-1;
	gdicommon_.more_rows           = (int)-1;
	gdicommon_.status              = (int)-1;


	/* dbVendor */
	gdicommon_.gdi_unknown         = (int)GDI_UNKNOWN;
	gdicommon_.gdi_sybase          = (int)GDI_SYBASE;
	gdicommon_.gdi_ingres          = (int)GDI_INGRES;
	gdicommon_.gdi_postgres        = (int)GDI_POSTGRES;
	gdicommon_.gdi_oracle          = (int)GDI_ORACLE;
	
#endif

	gdicommon_.gdi_noconn          = (int)0;
	gdicommon_.gdi_nomap           = (int)0;

	
	/* dbErrLEv */
	gdicommon_.gdi_noerror         = (int)GDI_NOERROR;
	gdicommon_.gdi_warning         = (int)GDI_WARNING;
	gdicommon_.gdi_fatal           = (int)GDI_FATAL;

	if (len > 0)
	{
		if ((pgname = UALLOC(char, len+1)) == (char *)NULL)
		{
			UFREE(pgname);
			return (GDI_FAILURE);
		}
		(void)gdi_for_to_cstr(pgname, len+1, prog_name, len);
	}
	
        i = gdi_init(pgname);
        UFREE(pgname);

	return (i);
        
}





