/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_f77_insert.c
 *
 * SYNOPSIS
 *
 *	#include "libgdi.h"
 *
 * DESCRIPTION
 *
 *	This file contains functions for the database Inserts
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *
 *	gdi_f77_insert.c (this file)
 *	gdi_insert.c 
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgdi.h
 *
 * AUTHOR
 *     H. Turner
 */


#include <stddef.h>
#include <stdlib.h>
#include "utilities.h"
#include "libgdi.h"
#include "f77defs.h"

#ifndef	lint
static char SccsId[] = "%W% %G%";
#endif


/*
 * gdi_insert ()
 *
 * Insert data into a database table
 *
 * Public
 */

int
gdi_insert (conn, table_name, datain)
int		*conn;
char		*table_name;
dbObj		*datain;
{
     int    ret;
     char   *fname = "gdi_insert: ";
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
	  return (GDI_SUCCESS);
     
     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->insert) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->insert (conn, table_name, datain)) != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_insert: Vendor function unavailable");

     return (GDI_FAILURE);
}

