/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_get_counter()
 *	
 *
 * DESCRIPTION
 *	This file contains functions for the specialized database capabilities.
 *
 * DIAGNOSTICS
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *	f77_gdi_spec.c (this file)
 *	gdi_spec.c
 *
 * SEE ALSO
 *	libgdi.h
 *
 * AUTHOR
 *	H. Turner
 */


#include "libgdi.h"
#include "utilities.h"
#include "f77defs.h"

#ifndef	lint
static char SccsId[] = "@(#)gdi_f77_spec.c	16.2 8/11/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif

/*
 * gdi_get_counter ()
 *
 * Get a unique value for a database counter.
 *
 * Public
 */

int
gdi_get_counter_ (conn, table_name, ctr_name, n_keys, key_value, 
		  table_len, ctr_len)
int	*conn;
char	*table_name;
char	*ctr_name;
int	*n_keys;
int	*key_value;
int     table_len;
int     ctr_len;
{
	char *fname = "gdi_get_counter_: ";
	dbConn *dbconn;
	char *table;
	char *ctr;
	int i;
	
	dbconn = get_db_conn(*conn);
	
	if ((table = UALLOC(char, table_len+1)) == (char *)NULL)
	{
		gdi_error_unix(dbconn, fname);
		UFREE(table);
		return (GDI_FAILURE);
	}
	(void)gdi_for_to_cstr(table, table_len+1, table_name, table_len);

	if ((ctr = UALLOC(char, ctr_len+1)) == (char *)NULL)
	{
		gdi_error_unix(dbconn, fname);
		UFREE(table);
		UFREE(ctr);
		return (GDI_FAILURE);
	}
	(void)gdi_for_to_cstr(ctr, ctr_len+1, ctr_name, ctr_len);
	
	i = gdi_get_counter(dbconn,
			    table,
			    ctr,
			    *n_keys,
			    key_value);
	UFREE(table);
	UFREE(ctr);

	return (i);
}

void
gdi_trace_ (conn, mode, filename, lname)
int	*conn;
int	*mode;
char    *filename;
int     lname;

{

	char *temp;
	
	dbConn *dbconn = get_db_conn(*conn);

	if ((temp = UALLOC(char, lname+1)) == (char *)NULL)
	{
		gdi_error_unix(dbconn, "gdi_trace_:");
		UFREE(temp);
		return;
		
	}
	(void)gdi_cstr_to_for(temp, lname+1, filename, lname);

	(void)gdi_trace(dbconn,
			*mode,
			temp);
	UFREE(temp);

}
