/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_submit.c
 *
 * SYNOPSIS
 *
 *	#include "libgdi.h"
 *
 *
 * DESCRIPTION
 *
 *	This file contains functions for the database Submits
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *
 *	gdi_f77_submit.c (this file)
 *	gdi_submit.c 
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgdi.h
 *
 * AUTHOR
 *	H. Turner
 */

#include "utilities.h"
#include "libgdi.h"
#include "f77defs.h"

#ifndef	lint
static char SccsId[] = "@(#)gdi_f77_submit.c	16.2 8/11/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif


/*
 * gdi_submit ()
 *
 * Submit a database command
 *
 * Public
 */

extern dbConstr F77_DEF_CONSTR;

int
gdi_submit_ (conn, map_id, cmd_batch, max_records, rows_retrieved, 
	     rows_affected, more_data, cmd_len)
int		*conn;
int             *map_id;
char		*cmd_batch;
int		*max_records;
int             *rows_retrieved;
int             *rows_affected;
int             *more_data;
int             cmd_len;

{
	char *cmd;
	
	dbConstr *constr = &F77_DEF_CONSTR;
	dbObj	 *results = (dbObj *)NULL;
	int      status;
	Col_Name_Addr *args;
	dbConn   *dbconn;
	
	
	dbconn = get_db_conn(*conn);
	
	if ((cmd = UALLOC(char, cmd_len+1)) == (char *)NULL)
	{
		gdi_error_unix(dbconn, "gdi_submit_:");
		UFREE(cmd);
		return (GDI_FAILURE);
	}
	(void)gdi_for_to_cstr(cmd, cmd_len+1, cmd_batch, cmd_len);

	args = retrieve_map(*map_id);
	args->row_count = 0;
	constr->args = (void *)args;
	
	status = gdi_submit(get_db_conn(*conn),
			    cmd,
			    *max_records,
			    constr,
			    &results);



	if (status == GDI_SUCCESS)
	{
		*rows_affected = results->query_info.rows_affected;
		*rows_retrieved = results->n_tuples;
		*more_data = results->query_info.more_rows;
	}
	else
	{
		*rows_affected = 0;
		*rows_retrieved = 0;
		*more_data = 0;
	}		
	UFREE(cmd);
	(void)gdi_obj_destroy(results);

	return (status);
}
