/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_commit()		end a transaction, making changes permanent
 *	gdi_rollback()		end a transaction, discarding changes
 *	gdi_savepoint()		set a savepoint (SYBASE & ORACLE)
 *
 * DESCRIPTION
 *	This file contains FORTRAN interfaces to functions for the 
 *      database transaction management.
 *
 * DIAGNOSTICS
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *	f77_gdi_tran.c (this file)
 *	gdi_tran.c
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgdi.h
 *
 * AUTHOR
 *	H. Turner
 */


#include "libgdi.h"
#include "utilities.h"
#include "f77defs.h"


#ifndef	lint
static char SccsId[] = "@(#)gdi_f77_tran.c	16.3 8/13/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif


/*
 * gdi_commit ()
 *
 * Commit a database transaction.
 *
 * Public
 */

int
gdi_commit_ (conn, channo, tran_name, tname_len)
int	*conn;
int	*channo;
char    *tran_name;
int     tname_len;
{
        char *tran;
	dbConn *dbconn;
	int  ret;
     
	dbconn = get_db_conn(*conn);
	
	if ((tran = UALLOC(char, tname_len+1)) == (char *)NULL)
	{
		gdi_error_unix(dbconn, "gdi_commit_:");
		return (GDI_FAILURE);
	}
	(void)gdi_for_to_cstr(tran, tname_len+1, tran_name, tname_len);

	ret = gdi_commit(dbconn, *channo, tran);

	UFREE (tran);
	return (ret);
}



/*
 * gdi_rollback ()
 *
 * Rollback a database transaction.
 *
 * Public
 */

int
gdi_rollback_ (conn, channo, tran_name, tname_len)
int	*conn;
int	*channo;
char    *tran_name;
int     tname_len;
{
        char *tran;
	dbConn *dbconn;
	int  ret;
     
	dbconn = get_db_conn(*conn);
	
	if ((tran = UALLOC(char, tname_len+1)) == (char *)NULL)
	{
		gdi_error_unix(dbconn, "gdi_rollback_:");
		return (GDI_FAILURE);
	}
	(void)gdi_for_to_cstr(tran, tname_len+1, tran_name, tname_len);

	ret = gdi_rollback(dbconn, *channo, tran);

	UFREE (tran);
	return (ret);
}



/*
 * gdi_savepoint ()
 *
 * Set a "save" point for a database transaction.
 *
 * Public
 */

int
gdi_savepoint_ (conn, channo, sname, sname_len)
int	*conn;
int	*channo;
char	*sname;
int     sname_len;
{

	char *save;
	dbConn *dbconn;
	int i;
	
	dbconn = get_db_conn(*conn);
	
	if ((save = UALLOC(char, sname_len+1)) == (char *)NULL)
	{
		gdi_error_unix(dbconn, "gdi_savepoint_:");
		UFREE(save);
		return (GDI_FAILURE);
	}
	(void)gdi_for_to_cstr(save, sname_len+1, sname, sname_len);

	i = gdi_savepoint(dbconn,
			  *channo,
			  save);
	UFREE(save);
	
	return (i);
	
}
