/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *      gdi_cstr_to_for() convert a C string to a FORTRAN array
 *	gdi_for_to_cstr() convert a FORTRAN array to a C string
 *
 * FILE
 *	gdi_for_to_c.c
 *
 * DESCRIPTION
 *	These routines convert between Null-terminated C strings and
 *	blank padded FORTRAN arrays.
 *
 *	If the length of the destination variable is larger than the size of
 *	the target variable, data are truncated.
 *
 * ARGUMENTS
 *	cstr		C source string
 *	c_length	length of the C string
 *	for_array	FORTRAN output array
 *	f_length	size of the FORTRAN array
 *
 * RETURN VALUE
 *	GDI_SUCCESS	no error
 *	GDI_FAILURE	The string was truncated when copied to the array
 *
 * AUTHOR
 *	J. T. Anderson	
 */

#ifndef	lint
static char SccsId[] = "@(#)gdi_for_to_c.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif

#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include "utilities.h"
#include "libgendb.h"

int
gdi_cstr_to_for(cstr, c_length, for_array, f_length)
char	*cstr;
int	c_length;
char	*for_array;
int	f_length;
{
	int	counter=0;			/* counter for blank padding */

		/* If the input string is larger than the array, truncate */
	if(c_length > f_length)
	{
		bcopy(cstr, for_array, f_length);
		return(GDI_FAILURE);
	}
		/* Otherwise copy and blank pad to the sizeof the array */
	else
	{
		bcopy(cstr, for_array, c_length);
		counter = c_length;

			/* Blank pad to the size of the array */
		while(counter < f_length)
		{
			for_array[counter] = ' ';
			counter++;
		}
		return(GDI_SUCCESS);
	}
}


int
gdi_for_to_cstr(cstr, c_length, for_array, f_length)
char	*cstr;
int	c_length;
char	*for_array;
int	f_length;
{
	int	numchar=0;	/* The number of characters to be copied */
	int	i;

		/* Insure that the target string is all spaces */
	for (i = c_length - 1; i >= 0; i--)
		cstr[i] = '\040';
	
		/* Remember that the last character of c_length must be NULL */
	if(f_length >= c_length)
		numchar=c_length;
	else
		numchar=f_length;

	bcopy(for_array, cstr, numchar);

	/* Starting at the end of the string, work backwards replacing white
	 * space with a NULL.  If the character is not a space the first time 
	 * this loop executes, but is real data, then the loop will terminate 
	 * and the string won't be NULL terminated.  So explicitely set the 
	 * last index NULL for those boundary cases. 
	 */
        for (i = c_length - 1; i > 0 && isspace(cstr[i]); i--)
		cstr[i] = '\0';

	if(cstr[c_length-1] != '\0')
	{
		cstr[c_length-1] = '\0';
		return(GDI_FAILURE);
	}
	else
		return(GDI_SUCCESS);
}
