/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_link
 *
 * SYNOPSIS
 *
 *	#include "libgendb.h"
 *
 * DESCRIPTION
 *	This file contains functions for managing the database connections
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *	gdi_link.c (this file)
 *
 * SEE ALSO
 *
 *	libgendb.h
 *
 * AUTHOR
 *	B. MacRitchie
 */


#include <stddef.h>
#include <stdlib.h>
#include <dld.h>
#include "utilities.h"
#include "libgendb.h"

#ifndef	lint
static char SccsId[] = "@(#)gdi_link.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif

Proto (void, gdi_reference_symbols, ());

#ifdef DEC

#ifdef ORACLE
Proto (dbFuncs *, ora_link, ());
#endif

#ifdef POSTGRES
Proto (dbFuncs *, pg_link, ());
#endif

#ifdef SYBASE
Proto (dbFuncs *, syb_link, ());
#endif

#endif

extern char *dld_errlst[];


int 
gdi_init (argv0)
char  *argv0;
{
     int    stat;
     char   *routine = "gdi_init: ";
     
#ifndef DEC
     if ((stat = dld_init (dld_find_executable (argv0))) != 0)
     {
	  gdi_error_app (NULL, GDI_BADDATA, dld_errlst [stat]);
	  gdi_error_preface (NULL, routine);
	  return (GDI_FAILURE);
     }
#endif
     return (GDI_SUCCESS);
}



int
gdi_link (conn)
dbConn  *conn;
{
     dbFuncs *(*link_fcn) () = NULL;
     
     if (conn == NULL) 
     {
	  gdi_error_app (conn, GDI_NOCONNECT,
			 "gdi_link: Invalid dbConn");
	  return (GDI_FAILURE);
     }

#ifndef DEC
     switch (conn->vendor_id)
     {
     case GDI_ORACLE:
	  link_fcn = (dbFuncs *(*) ()) dld_get_func ("ora_link");
	  break;
     case GDI_SYBASE:
	  link_fcn = (dbFuncs *(*) ()) dld_get_func ("syb_link");
	  break;
     case GDI_INGRES:
	  break;
     case GDI_POSTGRES:
	  link_fcn = (dbFuncs *(*) ()) dld_get_func ("pg_link");
	  break;
     default:
	  gdi_error_app (conn, GDI_BADVENDOR, "gdi_link: Unknown vendor");
	  return (GDI_FAILURE);
	  break;
     }
     
#else
     switch (conn->vendor_id)
     {
     case GDI_ORACLE:
#ifdef ORACLE
	  link_fcn = ora_link;
#endif
	  break;
     case GDI_SYBASE:
#ifdef SYBASE
	  link_fcn = syb_link;
#endif
	  break;
     case GDI_INGRES:
	  break;
     case GDI_POSTGRES:
#ifdef POSTGRES
	  link_fcn = pg_link;
#endif
	  break;
     default:
	  gdi_error_app (conn, GDI_BADVENDOR, "gdi_link: Unknown vendor");
	  return (GDI_FAILURE);
	  break;
     }
#endif

     if (link_fcn == NULL)
     {
	  gdi_error_app (conn, GDI_BADVENDOR, 
			 "gdi_link: Can't find vendor link function");
	  return (GDI_FAILURE);
     }

     GDI_FUNCS (conn) = link_fcn ();
     
     if (GDI_FUNCS (conn) == NULL)
     {
	  gdi_error_app (conn, GDI_BADVENDOR, "gdi_link: Vendor link failed");
	  return (GDI_FAILURE);
     }

     return (GDI_SUCCESS);
}



/*
 * This function references symbols that are not otherwise referenced in libgdi or
 * the application.  Although vendor libraries have been separated into their own 
 * libraries, they occasionally reference symbols in libgdi.  Since the vendor 
 * libraries are linked last, we need this function to resolve all symbols.
 */
void
gdi_reference_symbols ()
{
     void  *addr;
     
     addr = (void *) gdi_obj_col_def_add;
     addr = (void *) gdi_sleep;
     addr = (void *) gdi_col_def_create;
     addr = (void *) gdi_obj_create;
     addr = (void *) gdi_obj_tuple_retrieve;
     addr = (void *) gdi_obj_container_create;
     addr = (void *) gdi_obj_fill_data;
     addr = (void *) gdi_obj_destroy;
     addr = (void *) gdi_obj_get_data;
     addr = (void *) gdi_obj_tuple_add;
     addr = (void *) gdi_obj_num_columns;
     addr = (void *) gdi_obj_tuple_destroy;
     addr = (void *) gdi_obj_tuple_create;

     return;
}
