/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_sleep()
 *
 * SYNOPSIS
 *	#include "libgendb.h"
 *
 *	void
 *	gdi_sleep (conn, max_sleep)
 *	dbConn   *conn;		(i) Unix pid is stored here
 *	int  max_sleep;		(i) maximum number of seconds to sleep
 *
 * DESCRIPTION
 *	gdi_sleep() sleeps a random number of seconds that does not exceed
 *	max_sleep seconds. The sleep is random to de-synchronize the retries
 *	of processes pinging the same resource.
 *
 * FILES
 *	gdi_sleep.c (this file)
 *
 * SEE ALSO
 *	gdi_sleep(3)
 *
 * AUTHOR
 *	Jean T. Anderson, SAIC
 */

#include "stdio.h"
#include "libgendb.h"

#ifndef	lint
static char SccsId[] = "@(#)gdi_sleep.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif

Proto (int,	sleep, (unsigned seconds));
Proto (int,	getpid, ());
Proto (void,	srand48, (int seed));
Proto (long,	lrand48, ());

#ifndef DEC
Proto (int,	fprintf, ());
#endif

void
gdi_sleep(conn, max_sleep)
dbConn  *conn;
int max_sleep;
{
	static int	first=TRUE;

	if (max_sleep <= 0)
		return;

		/* Seed with the UNIX pid the first time this is called. */
	if (first == TRUE) {
		srand48 (GDI_OS_PID(conn));
		first = FALSE;
	}

	if (GDI_ERROR_DEBUG (conn))
		fprintf(stderr,
		"gdi_sleep: sleeping %d seconds\n", (lrand48() % max_sleep));

	sleep ( (lrand48 () % max_sleep) );
	return;
}
