/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_get_counter(), gdi_describe_object(), gdi_what_is_object(),
 *	gdi_describe_query(), gdi_trace()
 *	
 *
 * DESCRIPTION
 *	This file contains functions for the specialized database capabilities.
 *
 * DIAGNOSTICS
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *	gdi_spec.c (this file)
 *
 * SEE ALSO
 *	libgendb.h
 *
 * AUTHOR
 *	B. MacRitchie
 */


#include <stddef.h>
#include <stdlib.h>
#include "utilities.h"
#include "libgendb.h"

#ifndef	lint
static char SccsId[] = "@(#)gdi_spec.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif

/*
 * gdi_get_counter ()
 *
 * Get a unique value for a database counter.
 *
 * Public
 */

int
gdi_get_counter (conn, table_name, ctr_name, n_keys, key_value)
dbConn	*conn;
char	*table_name;
char	*ctr_name;
int	n_keys;
int	*key_value;
{
     int    ret;
     char   *fname = "gdi_get_counter: ";
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT,
			 "gdi_get_counter: Not connected to database");
	  return (GDI_FAILURE);
     }

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->get_counter) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->get_counter (conn, table_name, ctr_name, 
						    n_keys, key_value)) != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_get_counter: Vendor function unavailable");

     return (GDI_FAILURE);
}



/*
 * gdi_describe_object ()
 *
 * Describe a database object.
 *
 * Public
 */

int
gdi_describe_object (conn, object_name, constr, results)
dbConn	*conn;
char	*object_name;
dbConstr	*constr;
dbObj	**results;
{
     int    ret;
     char   *fname = "gdi_describe_object: ";
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT,
			 "gdi_describe_object: Not connected to database");
	  return (GDI_FAILURE);
     }

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->describe_object) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->describe_object (conn, object_name, constr, 
							results)) != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_describe_object: Vendor function unavailable");

     return (GDI_FAILURE);

}



/*
 * gdi_what_is_object ()
 *
 * Describe the database object.
 *
 * Public
 */

int
gdi_what_is_object (conn, object_name, constr, results)
dbConn	*conn;
char	*object_name;
dbConstr	*constr;
dbObj	**results;
{
     int    ret;
     char   *fname = "gdi_what_is_object: ";
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT,
			 "gdi_what_is_object: Not connected to database");
	  return (GDI_FAILURE);
     }

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->what_is_object) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->what_is_object (conn, object_name, constr, 
						       results)) != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_what_is_object: Vendor function unavailable");

     return (GDI_FAILURE);
}




/*
 * gdi_describe_query ()
 *
 * Describe result columns of the database query.
 *
 * Public
 */

int
gdi_describe_query (conn, query, results)
dbConn	*conn;
char	*query;
dbObj	**results;
{
     int    ret;
     char   *fname = "gdi_describe_query: ";
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT,
			 "gdi_describe_query: Not connected to database");
	  return (GDI_FAILURE);
     }

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->describe_query) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->describe_query (conn, query, results)) 
	      != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_describe_query: Vendor function unavailable");

     return (GDI_FAILURE);
}



int
gdi_trace (conn, mode, filename)
dbConn	*conn;
int	mode;
char	*filename;
{
     int    ret;
     char   *fname = "gdi_trace: ";
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT,
			 "gdi_trace: Not connected to database");
	  return (GDI_FAILURE);
     }

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->trace) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->trace (conn, mode, filename)) != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_trace: Vendor function unavailable");

     return (GDI_FAILURE);
}
