/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_submit.c
 *
 * SYNOPSIS
 *
 *	#include "libgendb.h"
 *
 *
 * DESCRIPTION
 *
 *	This file contains functions for the database Submits
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *
 *	gdi_submit.c (this file)
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgendb.h
 *
 * AUTHOR
 *	B. MacRitchie
 */


#include <stddef.h>
#include <stdlib.h>
#include "utilities.h"
#include "libgendb.h"

#ifndef	lint
static char SccsId[] = "@(#)gdi_submit.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif


/*
 * gdi_submit ()
 *
 * Submit a database command
 *
 * Public
 */

int
gdi_submit (conn, cmd_batch, max_records, constr, results)
dbConn		*conn;
char		*cmd_batch;
int		max_records;
dbConstr	*constr;
dbObj		**results;
{
     int    ret;
     char   *fname = "gdi_submit: ";
     
     if (results)
	  *results = NULL;
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT, 
			"gdi_submit: Database not connected");
	  return (GDI_FAILURE);
     }
     
     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->submit) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->submit (conn, cmd_batch, max_records, 
					       constr, results)) != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_submit: Vendor function unavailable");

     return (GDI_FAILURE);
}



