/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_temp.c
 *
 * SYNOPSIS
 *
 *	#include "libgendb.h"
 *
 *
 * DESCRIPTION
 *
 *	This file contains functions for managing temporary tables
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *
 *	gdi_temp.c (this file)
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgendb.h
 *
 * AUTHOR
 *	B. MacRitchie
 */


#include <stddef.h>
#include <stdlib.h>
#include "utilities.h"
#include "libgendb.h"


#ifndef	lint
static char SccsId[] = "@(#)gdi_temp.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif


/*
 * gdi_temp_name ()
 *
 * Determine full name for a temporary table
 *
 * Public
 */

int
gdi_temp_name (conn, name, new_name)
dbConn	*conn;
char	*name;
char	*new_name;
{
     int    ret;
     char   *fname = "gdi_temp_name: ";
     
     if ((conn == NULL) || (conn->vendor_conn == NULL))
     {
	  gdi_error_app (conn, GDI_NOCONNECT, 
			"gdi_temp_name: Not connected to database");
	  return (GDI_FAILURE);
     }

     if ((GDI_FUNCS (conn) != NULL) && (GDI_FUNCS (conn)->temp_name) != NULL)
     {
	  if ((ret = GDI_FUNCS (conn)->temp_name (conn, name, new_name)) != GDI_SUCCESS)
	       gdi_error_preface (conn, fname);
	  return (ret);
     }

     gdi_error_app (conn, GDI_BADVENDOR, 
		    "gdi_temp_name: Vendor function unavailable");

     return (GDI_FAILURE);
}




