/* See the copyright notice (COPYRIGHT) in this directory. */

/*
 * NAME
 *	gdi_turbo.c
 *
 * SYNOPSIS
 *
 *	#include "libgendb.h"
 *
 *
 * DESCRIPTION
 *
 *	This file contains functions for creating the streamlined turboTup
 *	of the Generic Database Interface.
 *
 * DIAGNOSTICS
 *
 *	Functions which return pointers will return NULL if an error
 *	occurred.  Functions which return a non-negative int will return 
 *	-1 if an error occurred.
 *
 * FILES
 *
 *	gdi_turbo.c (this file)
 *
 * NOTES
 *
 *
 * SEE ALSO
 *
 *	libgendb.h
 *
 * AUTHOR
 */


#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include "utilities.h"
#include "libgendb.h"
#include "gdi_turbo.h"
#include "proto.h"


#ifndef	lint
static char SccsId[] = "@(#)gdi_turbo.c	16.1 8/3/93 Copyright (c) 1992-1993 Science Applications International Corporation";
#endif

/*
 * macros for Access of default dbTuple
 */

#define GDI_TURBO_TUPLE_NUM_COLUMNS(tuple)	 ((tuple)->n_columns)
#define GDI_TURBO_TUPLE_NAME(tuple, col_num)	 ((tuple)->col_defs[col_num]->name)
#define GDI_TURBO_TUPLE_DBTYPE_S(tuple, col_num) ((tuple)->col_defs[col_num]->dbtype_s)
#define GDI_TURBO_TUPLE_CTYPE(tuple, col_num)	 ((tuple)->col_defs[col_num]->ctype)
#define GDI_TURBO_TUPLE_LENGTH(tuple, col_num)	 ((tuple)->col[col_num].str_length)
#define GDI_TURBO_TUPLE_STR_LENGTH(tuple, col_num) ((tuple)->col[col_num].str_length)
#define GDI_TURBO_TUPLE_ARR_LENGTH(tuple, col_num) ((tuple)->col[col_num].arr_length)
#define GDI_TURBO_TUPLE_IS_NULL(tuple, col_num)	 ((tuple)->col[col_num].is_null)
#define GDI_TURBO_TUPLE_VALUE(tuple, col_num)	 (&((tuple)->col[col_num].value))

/*
 * prototypes for turbo Tuple Constructor
 */

Proto (static turboTup *, gdi_turbo_tuple_create, 
       (
	dbColDef **col_def, 		/* (i) column defintion */
	void     **args                 /* (i) pointer to constructor arguments */
	));

Proto (static int, gdi_turbo_tuple_destroy, 
       (
	turboTup *tuple, 		/* (i) turbo tuple       */
	void     **args                 /* (i) pointer to constructor arguments */
	));

Proto (static int, gdi_turbo_tuple_fill_value, 
       (
	int      col_num, 		/* (i) column number     */
	dbColDef *col_def, 		/* (i) column definition */
	turboTup *tuple, 		/* (i) turbo tuple       */
	void     *dbdata, 		/* (i) database data     */
	int	 is_null, 		/* (i) "is null" flag    */
	int      str_length,		/* (i) string length (including NULL term) */
	int      arr_length,		/* (i) array length      */
	void     **args                 /* (i) pointer to constructor arguments    */
	));

Proto (static void *, gdi_turbo_tuple_get_value, 
       (
	int      col_num, 		/* (i) column number     */
	dbColDef *col_def, 		/* (i) column definition */
	turboTup *tuple, 		/* (i) turbo tuple       */
	int      *is_null, 		/* (o) "is null" flag    */
	int      *str_length,		/* (o) string length (including NULL term) */
	int      *arr_length,		/* (o) array length     */
	void     **args                 /* (i) pointer to constructor arguments    */
	));

Proto (static int, gdi_turbo_tuple_add, 
       (
	turboTup ***tarray,		/* (i) turbo data container */
	turboTup *tuple, 		/* (i) turbo tuple          */
	void     **args                 /* (i) pointer to constructor arguments */
	));

Proto (static turboTup *, gdi_turbo_tuple_retrieve, 
       (
	turboTup **tarray, 		/* (i) data container  */
	int      tuple_num,		/* (i) tuple number    */
	void     **args                 /* (i) pointer to constructor arguments */
	));


Proto (static int, fill_scalar,
       (
	dbValue   *value,               /* (i) tuple value   */
	void      *dbdata,              /* (i) database data */
	dbCtype   ctype                 /* (i) C type        */
	));

Proto (static int, fill_string,
       (
	dbValue   *value,               /* (i) tuple value   */
	void      *dbdata,              /* (i) database data */
	int       str_length            /* (i) string length */
	));

Proto (static int, fill_scalar_array,
       (
	dbValue   *value,               /* (i) tuple value   */
	void      *dbdata,              /* (i) database data */
	dbCtype   ctype,                /* (i) C type        */
	int       arr_length            /* (i) array length  */
	));

Proto (static int, fill_string_array,
       (
	dbValue   *value,               /* (i) tuple value   */
	void      *dbdata,              /* (i) database data */
	int       str_length,           /* (i) string length */
	int       arr_length            /* (i) array length  */
	));



#ifdef __STDC__
dbConstr	GDI_TURBO = 
{
     (int    (*) ())                            NULL,
     (void * (*) ())			        NULL,
     (int    (*) (void **, void *, void **))	gdi_turbo_tuple_add,
     (void * (*) (void *, int, void **))	gdi_turbo_tuple_retrieve,
     (int    (*) (void *, void **))		gdi_turbo_tuple_destroy,
     (void * (*) (dbColDef **, void **))        gdi_turbo_tuple_create,
     (int    (*) (int, dbColDef *, void *, void *, int, int, int, void **))
	                                        gdi_turbo_tuple_fill_value,
     (void * (*) (int, dbColDef *, void *, int *, int *, int *, void **))
	                                        gdi_turbo_tuple_get_value,
     (int    (*) ())                            NULL,
     (void *)	NULL,
     (dbCoerce)	GDI_DEFAULT_COERCE,
};
#else
dbConstr	GDI_TURBO = 
{
     (int    (*) ())	NULL,
     (void * (*) ())	NULL,
     (int    (*) ())	gdi_turbo_tuple_add,
     (void * (*) ())	gdi_turbo_tuple_retrieve,
     (int    (*) ())	gdi_turbo_tuple_destroy,
     (void * (*) ())	gdi_turbo_tuple_create,
     (int    (*) ())	gdi_turbo_tuple_fill_value,
     (void * (*) ()) 	gdi_turbo_tuple_get_value,
     (int    (*) ())	NULL,
     (void *)	NULL,
     (dbCoerce)	GDI_DEFAULT_COERCE
};
#endif



/*
 * gdi_turbo_tuple_add ()
 *
 * Add a tuple to the tuple array.
 *
 * Private
 */
static int
gdi_turbo_tuple_add (tarray, tuple, args)
turboTup    ***tarray;
turboTup    *tuple;
void        **args;
{
     int	i;
     turboTup	**t;
     char	*routine = "gdi_turbo_tuple_add: ";
     
     if (tarray == NULL)
	  return (GDI_FAILURE);
     
     for (i = 0; (*tarray != NULL) && (*tarray)[i] != NULL; i++);

     if ((t = UREALLOC ((*tarray), turboTup *, i + 2)) == NULL)
     {
	  gdi_error_unix (NULL, routine);
	  return (GDI_FAILURE);
     }
     
     (*tarray) = t;
     
     (*tarray) [i] = tuple;
     (*tarray) [i+1] = NULL;

     return (GDI_SUCCESS);
}


/*
 * gdi_turbo_tuple_retrieve ()
 *
 * Retrieve a tuple from the tuple array.
 *
 * Private
 */
static turboTup *
gdi_turbo_tuple_retrieve (tarray, tuple_num, args)
turboTup	**tarray;
int		tuple_num;
void            **args;
{
     if (tarray == NULL)
	  return (NULL);
     
     return (tarray [tuple_num]);
}


/*
 * gdi_turbo_tuple_destroy ()
 *
 * Destroy a tuple.
 *
 * Private
 */
static int
gdi_turbo_tuple_destroy (tuple, args)
turboTup	*tuple;
void            **args;
{
     dbCtype    ctype;
     int        arr_length;
     int	n;
     int	i;
     
     if (tuple == NULL)
	  return (GDI_SUCCESS);
     
     for (n = 0; n < GDI_TURBO_TUPLE_NUM_COLUMNS(tuple); n++)
     {
	  ctype = GDI_TURBO_TUPLE_CTYPE (tuple, n);
	  arr_length = GDI_TURBO_TUPLE_ARR_LENGTH (tuple, n);
	  
	  if ((ctype == GDI_STRING) && (arr_length == GDI_SCALAR))
	  {
	       /* free string */
	       UFREE (GDI_TURBO_TUPLE_VALUE (tuple, n)->string);
	  }
	  else if ((ctype == GDI_STRING) && (arr_length > GDI_SCALAR))
	  {
	       /* free array of strings */
	       for (i = 0; i < arr_length; i++)
		    UFREE (((char **) GDI_TURBO_TUPLE_VALUE (tuple, n)->ptr) [i]);

	       UFREE (GDI_TURBO_TUPLE_VALUE (tuple, n)->ptr);
	  } 
	  else if (arr_length > GDI_SCALAR)
	  {
	       /* free array of scalars */
	       UFREE (GDI_TURBO_TUPLE_VALUE (tuple, n)->ptr);
	  }
	  
     }
     
     UFREE (tuple->col);
     UFREE (tuple);

     return (GDI_SUCCESS);
}     
	  
     
/*
 * gdi_turbo_tuple_create ()
 *
 * Create a new tuple.
 *
 * Private
 */
static turboTup *
gdi_turbo_tuple_create (coldef, args)
dbColDef	**coldef;
void		**args;
{
     int	n;
     turboTup   *tuple;
     char	*routine = "gdi_turbo_tuple_create: ";
     
     if (coldef == NULL)
	  return (NULL);
     
     if ((tuple = UCALLOC (turboTup, 1)) == NULL)
     {
	  gdi_error_unix (NULL, routine);
	  return (NULL);
     }

     tuple->col_defs = coldef;

     for (n = 0; coldef[n] != NULL; n++);
     if ((tuple->col = UCALLOC (turboCol, n)) == NULL)
     {
	  gdi_error_unix (NULL, routine);
	  UFREE (tuple);
	  return (NULL);
     }
     
     GDI_TURBO_TUPLE_NUM_COLUMNS(tuple) = n;

     return (tuple);
}


/*
 * gdi_turbo_tuple_fill_value ()
 *
 * Fill a tuple column value with database data.  Length for a string
 * may or may not include a NULL terminator.
 *
 * Private
 */
static int
gdi_turbo_tuple_fill_value (col_num, col_def, tuple, dbdata, is_null, 
			    str_length, arr_length, args)
int		col_num;
dbColDef	*col_def;
turboTup	*tuple;
void		*dbdata;
int		is_null;
int		str_length;
int		arr_length;
void            **args;
{
     dbCtype    ctype;
     int        ret;
     
     if (col_num < 0)
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_turbo_tuple_fill_value: invalid column number");
	  return (GDI_FAILURE);
     }
     
     if ((col_def == NULL) || (tuple == NULL) || (dbdata == NULL))
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_turbo_tuple_fill_value: NULL pointer argument");
	  return (GDI_FAILURE);
     }
     
     GDI_TURBO_TUPLE_IS_NULL (tuple, col_num) = is_null;
     GDI_TURBO_TUPLE_STR_LENGTH (tuple, col_num) = str_length;
     GDI_TURBO_TUPLE_ARR_LENGTH (tuple, col_num) = arr_length;

     ctype = GDI_TURBO_TUPLE_CTYPE (tuple, col_num);
     if ((arr_length == GDI_SCALAR) && (ctype != GDI_STRING))
     {
	  ret = fill_scalar (GDI_TURBO_TUPLE_VALUE (tuple, col_num), 
			     dbdata, ctype);
     }
     else if ((arr_length == GDI_SCALAR) && (ctype == GDI_STRING))
     {
	  ret = fill_string (GDI_TURBO_TUPLE_VALUE (tuple, col_num), 
			     dbdata, str_length);
     }
     else if (ctype != GDI_STRING)
     {
	  ret = fill_scalar_array (GDI_TURBO_TUPLE_VALUE (tuple, col_num), 
				   dbdata, ctype, arr_length);
     }
     else
     {
	  ret = fill_string_array (GDI_TURBO_TUPLE_VALUE (tuple, col_num), 
				   dbdata, str_length, arr_length);
     }
     
     return (ret);
}     


/*
 * fill_scalar ()
 *
 * Fill a tuple column value with a scalar database data.
 *
 * Private
 */
static int
fill_scalar (value, dbdata, ctype)
dbValue         *value;
void            *dbdata;
dbCtype         ctype;
{
     switch (ctype)
     {
     case GDI_CHAR:
	  value -> char_val = * ((char *) dbdata);
	  break;
     case GDI_SHORT:
	  value -> short_val = * ((short *) dbdata);
	  break;
     case GDI_INTEGER:
	  value -> int_val = * ((int *) dbdata);
	  break;
     case GDI_LONG:
	  value -> long_val = * ((long *) dbdata);
	  break;
     case GDI_UCHAR:
	  value -> uchar_val = * ((unsigned char *) dbdata);
	  break;
     case GDI_USHORT:
	  value -> ushort_val = * ((unsigned short *) dbdata);
	  break;
     case GDI_UINTEGER:
	  value -> uint_val = * ((unsigned int *) dbdata);
	  break;
     case GDI_ULONG:
	  value -> ulong_val = * ((unsigned long *) dbdata);
	  break;
     case GDI_FLOAT:
	  value -> float_val = * ((float *) dbdata);
	  break;
     case GDI_DOUBLE:
	  value -> double_val = * ((double *) dbdata);
	  break;
     default:
	  break;
     }

     return (GDI_SUCCESS);
}

	  

/*
 * fill_string ()
 *
 * Fill a tuple column value with a string database data.
 *
 * Private
 */
static int
fill_string (value, dbdata, str_length)
dbValue         *value;
void            *dbdata;
int             str_length;
{
     char       *routine = "gdi_turbo_fill_string: ";
     
     /* allocate extra space to add our own NULL terminator */
     if ((value -> string = UCALLOC (char, str_length + 1)) == NULL)
     {
	  gdi_error_unix (NULL, routine);
	  return (GDI_FAILURE);
     }
     /* 
      * use bcopy instead of strncpy in case string contains
      * embedded NULLs.
      */
     bcopy ((char *) dbdata, value -> string, str_length * sizeof (char));
     value -> string [str_length] = '\0';
     
     return (GDI_SUCCESS);
}



/*
 * fill_scalar_array ()
 *
 * Fill a tuple column value with a scalar array database data.
 *
 * Private
 */
static int
fill_scalar_array (value, dbdata, ctype, arr_length)
dbValue         *value;
void            *dbdata;
dbCtype         ctype;
int             arr_length;
{
     int        csize;
     char       *routine = "gdi_turbo_fill_scalar_array: ";

     switch (ctype)
     {
     case GDI_CHAR:
     case GDI_UCHAR:
	  csize = sizeof (char);
	  break;
     case GDI_SHORT:
     case GDI_USHORT:
	  csize = sizeof (short);
	  break;
     case GDI_INTEGER:
     case GDI_UINTEGER:
	  csize = sizeof (int);
	  break;
     case GDI_LONG:
     case GDI_ULONG:
	  csize = sizeof (long);
	  break;
     case GDI_FLOAT:
	  csize = sizeof (float);
	  break;
     case GDI_DOUBLE:
	  csize = sizeof (double);
	  break;
     default:
	  csize = 0;
	  break;
     }
     
     if ((value -> ptr = (void *) UCALLOC (char *, csize * arr_length)) == NULL)
     {
	  gdi_error_unix (NULL, routine);
	  return (GDI_FAILURE);
     }
     
     bcopy ((char *) dbdata, value -> ptr, csize * arr_length);

     return (GDI_SUCCESS);
}     
     

/*
 * fill_string_array ()
 *
 * Fill a tuple column value with a string array database data.
 *
 * Private
 */

static int
fill_string_array (value, dbdata, str_length, arr_length)
dbValue         *value;
void            *dbdata;
int             str_length;
int             arr_length;
{
     int        i;
     char       *routine = "gdi_turbo_fill_string_array: ";
     
     if ((value -> ptr = (void *) UCALLOC (char *, arr_length)) == NULL)
     {
	  gdi_error_unix (NULL, routine);
	  return (GDI_FAILURE);
     }

     for (i = 0; i < arr_length; i++)
     {
	  if ((((char **) value -> ptr) [i] = UCALLOC (char, str_length + 1)) == NULL)
	  {
	       gdi_error_unix (NULL, routine);
	       for (i--; i >= 0; i--)
		    UFREE (((char **) value -> ptr) [i]);
	       UFREE (value -> ptr);
	       return (GDI_FAILURE);
	  }
	  
	  bcopy (((char **) dbdata) [i], ((char **) value -> ptr) [i], 
		 str_length * sizeof (char));
	  ((char **) value -> ptr) [i] [str_length] = '\0';
      }

     return (GDI_SUCCESS);
}
	  


/*
 * gdi_turbo_tuple_get_value ()
 *
 * Return a tuple column value.  Also fill in is_null and length.
 *
 * Private
 */
static void *
gdi_turbo_tuple_get_value (col_num, col_def, tuple, is_null, 
			   str_length, arr_length, args)
int		col_num;
dbColDef	*col_def;
turboTup	*tuple;
int		*is_null;
int		*str_length;
int		*arr_length;
void            **args;
{
     void	*dbdata;
     
     if (col_num < 0)
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_turbo_tuple_get_value: invalid column number");
	  return (NULL);
     }
     
     if (tuple == NULL)
     {
	  gdi_error_app (NULL, GDI_BADDATA,
			 "gdi_turbo_tuple_get_value: NULL pointer argument");
	  return (NULL);
     }
     
     if (is_null != NULL)
	  *is_null = GDI_TURBO_TUPLE_IS_NULL (tuple, col_num);
     
     if (str_length != NULL)
	  *str_length = GDI_TURBO_TUPLE_STR_LENGTH (tuple, col_num);
     
     if (arr_length != NULL)
	  *arr_length = GDI_TURBO_TUPLE_ARR_LENGTH (tuple, col_num);
     
     if (col_def->ctype == GDI_STRING)
	  dbdata = (void *) GDI_TURBO_TUPLE_VALUE (tuple, col_num)->string;
     else
	  dbdata = (void *) GDI_TURBO_TUPLE_VALUE (tuple, col_num);
     
     return (dbdata);
}




