#
# ScanActionDirs - calls AddActionDir for each directory in PGB(ActionDirs)
#
proc ScanActionDirs {} {
	global PGB
	foreach dir [ActionInfo dirs] {
		if {![file isdirectory $dir] || ![file readable $dir]} continue
		ActionInfo adddir $dir
		foreach adir [glob $dir/*] {
			set action [file tail $adir]
			if ![file readable $adir/$PGB(ActionInit)] continue
			set Name $action
			set Help {No description available.}
			# an ActionInit file generally sets Name and Help to
			# something nice and contains an "Action" procedure
			source $adir/$PGB(ActionInit)
			ActionInfo addname $dir $action $Name
			ActionInfo addhelp $dir $action $Help
		}
	}
}

#
# ActionInfo -	utility procedure to create and access Action arrays
#
# inputs:
#	cmd	command, one of: dirs, adddir, addname, addhelp, or button
#	args	command args
#
# returns:
#	if not adding something, the action directories or
#	or a list suitable for a mkMenuBar button entry
#
proc ActionInfo {cmd args} {
	global PGB PGBActionName PGBActionHelp
	case $cmd in {
		dirs	{
			if [info exists PGB(ActionDirs)] {
				return [lsort $PGB(ActionDirs)]
			} else {
				return {}
			}
			}
		adddir	{
			# create new arrays for an actions directory 
			set dir [lindex $args 0]
			if {$dir == {}} {
				error {Wrong # args, should be:  "ActionInfo adddir dir"}
			}
			if ![info exists PGB(NumActionDirs)] {set PGB(NumActionDirs) 1}
			set PGBActionName($dir) PGB_GN$PGB(NumActionDirs)
			set PGBActionHelp($dir) PGB_GH$PGB(NumActionDirs)
			incr PGB(NumActionDirs) 1
			}
		addname	{
			if {[llength $args] != 3} {
				error {Wrong # args, should be:  "ActionInfo addname dir action name"}
			}
			set dir [lindex $args 0]
			set action [lindex $args 1]
			set name [lindex $args 2]
			upvar #0 $PGBActionName($dir) Name
			set Name($action) $name
			}
		addhelp {
			if {[llength $args] != 3} {
				error {Wrong # args, should be:  "ActionInfo addhelp dir action help"}
			}
			set dir [lindex $args 0]
			set action [lindex $args 1]
			set help [lindex $args 2]
			upvar #0 $PGBActionHelp($dir) Help
			set Help($action) $help
			}
		button	{
			set dir [lindex $args 0]
			if {$dir == {}} {
				error {Usage:  "ActionInfo button dir"}
			}
			upvar #0 $PGBActionName($dir) Name
			upvar #0 $PGBActionHelp($dir) Help
			foreach action [lsort [array names Name]] {
				set name $Name($action)
				set help $Help($action)
				lappend invoke DoAction $dir/$action
				lappend e $name $invoke $help
				lappend r $e
				unset e invoke
			}
			return $r
			}
		default {
			error "No such command: $cmd"
			}
	}
}

#
# AddActionButton - adds an action button to a mkMenuBar menu
#
# input:
#	w	mkMenuBar frame
#
# action:
#	create an Action menubutton and then for each action directory
#	in [ActionInfo dirs], add a cascaded menu for each action and
#	add cascade help to the Help button.  When an action is selected,
#	DoAction is called with the action directory.
#
proc AddActionButton { w } {
        global PGB errorInfo
	if ![winfo exists $w] {
		error "mkMenuBar frame $w doesn't exist."
	}
	menubutton $w.actions -menu $w.actions.m -text Actions
	menu $w.actions.m
	if ![winfo exists $w.help] {
		menubutton $w.help -label Help -menu $w.help.m
		menu $w.help.m
		pack append $w $w.help right
	}
        $w.help.m add cascade -label "Actions..." -menu $w.help.m.actions
        menu $w.help.m.actions
	set n 0
	foreach dir [ActionInfo dirs] {
		set shortname [file tail $dir]
                $w.help.m.actions add cascade -label "$shortname..." \
                        -menu $w.help.m.actions.$n
                menu $w.help.m.actions.$n
		$w.actions.m add cascade -label "$shortname..." \
			-menu $w.actions.m.$n
		menu $w.actions.m.$n
		foreach action [ActionInfo button $dir] {
			set name [lindex $action 0]
			set cmd [lindex $action 1]
			set help [lindex $action 2]
			$w.actions.m.$n add command \
				-label $name -command $cmd
                        $w.help.m.actions.$n add command -label $name \
                                -command "HelpBox \"$help\""
		}
		incr n 1
	}
	pack append $w $w.actions left
}

#
# DoAction -	execute an action
#
# inputs:
#	actiondir
#
proc DoAction { actiondir } {
	global PGB
	set init $actiondir/$PGB(ActionInit)
	if {![file readable $init]} {
		ErrorBox "Can't run action in directory:  $actiondir"
		return 0
	}
	# remove current Action
	proc Action {} {}
	# source new Action
	source $init
	if {[info body Action] == {}} {
		ErrorBox "No Action procedure defined in $init, can't run action."
		return 0
	}
	# run Action
	return [Action $actiondir]
}

proc SelectFields { } {
	global Columns
	toplevel .s
	label .s.l -text "Select Columns"
	pack append .s .s.l top
	set n 0
	foreach a [Attribute nameorder] {
		set Columns($n) 0
		checkbutton .s.$n -variable Columns($n) \
			-text $a -relief flat 
		pack append .s .s.$n {top frame w}
		incr n 1
	}
	mkButton .s.c Cancel {set Columns(return) 0;destroy .s}
	mkButton .s.b OK {set Columns(return) 1;destroy .s}
	pack append .s .s.b left .s.c right
	tkwait visibility .s
	grab .s
	tkwait window .s
	for {set i 0} {$i < $n} {incr i 1 } {
		if $Columns($i) {lappend r $i}
	}
	if {!$Columns(return) || ![info exists r]} {
		unset Columns ; return {}
	}
	unset Columns
	return $r
}


proc SelectAField { } {
	global ColumnNum
	toplevel .s
	label .s.l -text "Select a Column"
	pack append .s .s.l top
	set ColumnNum 0
	set n 0
	foreach a [Attribute nameorder] {
		radiobutton .s.$n -variable ColumnNum -value $n \
			-text $a -relief flat 
		pack append .s .s.$n {top frame w}
		incr n 1
	}
	mkButton .s.c Cancel {set ColumnNum -1;destroy .s}
	mkButton .s.b OK {destroy .s}
	pack append .s .s.b left .s.c right
	tkwait visibility .s
	grab .s
	tkwait window .s
	set n $ColumnNum
	unset ColumnNum
	return $n
}

proc ReturnAField { } {
	global PGB
	if {[set n [SelectAField]] == -1} return {}
	return [GetAField $n]
}

proc GetAField { n } {
	global ResultsCol
#	.qr.$n.ls.l select from 0
#	.qr.$n.ls.l select to end
#	return [selection get]
	return $ResultsCol($n)
}
