#
# Procedures used by PGBROWSE
#
# Bugs?  Wirdness?  E-mail jimbo@crseo.ucsb.edu
# Apr. 1993
#

#
# ReadPGTypes -	read pgtypes.dat file and fill arrays used by TypeInfo
#
# action:
#	fills in arrays maintained by TypeInfo or kills pgbrowse
#	if pgtypes.dat doesn't exist
#
proc ReadPGTypes { } {
	global PGB

	set file $PGB(Home)/lib/pgtypes.dat
	if [catch {set fd [open $file r]} err] {
		ErrorBox $err
		destroy .; exit 1
	}

	while { ![eof $fd] } {
		set line [gets $fd]
		case $line in {
			{#* {}} {}
			default {
					set line [split $line \t]
					TypeInfo add [lindex $line 0] [lindex $line 1]
				}
		}
	}
	close $fd
}

#
# TypeInfo -	type name/description access procedure
#
# input:
#	cmd	what to return
#	args	command args
#
proc TypeInfo {cmd args} { 
        global PGBTNameByDesc PGBTDescByName
        
	case $cmd in {

                names	{return [array names PGBTDescByName]}

                descs	{return [array names PGBTNameByDesc]}

	 	namea*	{return PGBTNameByDesc}

	 	desca*	{return PGBTDescByName}

		name	{
				set desc [lindex $args 0]
				if ![info exists PGBTNameByDesc($desc)] {
					return {}
				} else {
					return $PGBTNameByDesc($desc)
				}
			}

        	desc	{
				set name [lindex $args 0]
				set array ""
				if [regsub ^_ $name "" name] {
					set array "array of "
				}
				if ![info exists PGBTDescByName($name)] {
					return {}
				} else {
					return "$array$PGBTDescByName($name)"}
				}

		add	{
				set name [lindex $args 0]
				set desc [lindex $args 1]
				set PGBTDescByName($name) $desc
				set PGBTNameByDesc($desc) $name
			}

		default {
				error "TypeInfo: unknown command: $cmd"
			}
	}
}

#
# QuitPGB -	Exit pgrbrowse
#
# action:
#	Ask user to save log before deleting it and then exit
#
proc QuitPGB { } {
	global PGB
	case [SelectAButton "Save log file before exiting?" \
		{{Yes, save log}} {{No, abandon log}} {{Continue PGBROWSE}}] in {
		0	{
				if ![SaveLog] return
				# remove temporary log file
				if !$PGB(UserSpecifiedLog) {
					catch {close $PGB(Logfile)}
					exec rm -f $PGB(Logfile)
				}
			}
		1	{ 
				# remove temporary log file
				catch {close $PGB(Logfile)}
				exec rm -f $PGB(Logfile)
			}
		2	return
	}
	destroy .
	exit 0
}


#
# DoMonitor -	send a command to the Postgres backend via. monitor
#
# inputs:
#	args	variable list of keys:
#		-flags flags for monitor (default: -N -T -Q)
#		-return name of variable to place results (default: DMReturn)
#		-nolog	boolean, don't log result
#               -file store results in a temporary file instead of the
#		return string
# returns:
#	1	success, -return holds result
#	0	error, -return holds error message
#
proc DoMonitor { args } {
	global PGB

	set flags {-N -T -Q}
	set return DMReturn
	if [catch {set i [ParseArgs {file nolog :flags :return} $args]}] {
		return 0
	}

	upvar 1 $return r

        set sucess "Return: SUCCESS:"
        set error "Return: ERROR:"
	set command "PostQuel Command:"

	set cmd "{[string trim [lindex $args $i]]}"
	set monitor "$PGB(Monitor) $flags -h $PGB(Host) -c $cmd $PGB(Db)"

	Working
	if $file {
		set filename $PGB(Tmp)/monitor.[GetAPid]
		set monitor "$monitor > $filename"
		AddToLog $monitor $command
		if [catch {eval exec $monitor} err] {
			set r $err
			AddToLog $r $error
			set status 0
		} else {
                	AddToLog "(output in file: $filename)" $sucess
			set r $filename
			set status 1
		}
	} else {
		set monitor "|$monitor"
		AddToLog $monitor $command
		set fd [open "$monitor" r]
		set mr [read $fd]
		if {[catch { close $fd } err] != 0} {
			set r $err
			AddToLog $r $error
			set status 0
		} else {
			if $nolog {
				AddToLog "(output not logged)" $sucess
			} else {
				AddToLog $mr $sucess
			}
			foreach line [split $mr \n] {
				if {$line != {}} {lappend r [string trim $line]}
			}
			set status 1
		}
	}
	Waiting
	return $status
}

#
# AddToLog -	adds a message to the log window, used by DoMonitor and
#		Query* procedures
#
# input:
#	msg	message to add
#	code	code message before message, e.g., SUCCESS or ERROR or
#		something else sensible
#
# action:
#	adds line to log file and, if it exists, log window.  NB: if the
#	log window exists we could just skip the log file because CloseLog
#	will sync the log file with the log window contents before closing
#	but I thought a current log file would be helpful if there's a
#	crash.  If logging gets slow we could add some variables to
#	specify the action (like DontSync to keep CloseLog from syncing).
#	What the hell, things seem fast enough as is.
#
proc AddToLog { msg code } {
	global PGB
	set log "$code\n$msg\n\n"
	puts $PGB(Logfd) $log
	if [winfo exists .showlog.text] {
		.showlog.text insert end $log
		.showlog.text yview -pickplace insert
		update
	}
}
#
# OpenLog -	Open the log window
#
# action:
#	read current logfile contents into an editable text window.
#
proc OpenLog { } {
	global PGB
	if [winfo exists .showlog] return
	seek $PGB(Logfd) 0
	toplevel .showlog
	wm title .showlog "Postquel Command Log:  $PGB(Logfile)"
	text .showlog.text -yscroll {.showlog.sb set} -bd 2 -relief sunken
	scrollbar .showlog.sb -command {.showlog.text yview}
	.showlog.text insert insert [read $PGB(Logfd)]
	.showlog.text tag configure bold -font -Adobe-Courier-Bold-O-Normal-*-120-*
	mkMenuBar .showlog.menu {
		{About {This window displays the commands sent to the Postgres backend and the messages returned.  You can save the log at anytime into a flat ascii text file.  Also, you can add notes to the log by simply clicking at the desired location and typing - your notes will be saved along with the Postquel dialog.}}
	} {
		File {	
			{{Save Log} SaveLog {Save the current Postquel log to a file.}}
			{Close CloseLog {Closes the log window and writes any edits to disk.}}
		}
	}
	mkInfoBar .showlog.info
	pack append .showlog \
		.showlog.menu {top fillx} \
		.showlog.info {top fillx} \
		.showlog.text left \
		.showlog.sb {left filly}
}

#
# CloseLog -	calls SyncLog and closes window
#
proc CloseLog { } {
	global PGB
	if ![winfo exists .showlog] return
	SyncLog
	destroy .showlog
}

#
# SyncLog -	syncs log window and file
#
# action:
#	seek's logfile to beginning, copies log window contents to
#	it, and then closes the window.  See note in "AddToLog"
#
proc SyncLog { } {
	global PGB
	if ![winfo exists .showlog] return
	seek $PGB(Logfd) 0
	puts $PGB(Logfd) [.showlog.text get 0.0 end]
}

#
# SaveLog -	prompts user for and then saves to a log file
#
proc SaveLog { } {
	global PGB
	if [winfo exists .showlog.text] SyncLog
	if $PGB(UserSpecifiedLog) {return 1}
	if {[set fd [FDSave "Save Postquel Log" -file pg.log]] == ""} {return 0}
	flush $PGB(Logfd)
	seek $PGB(Logfd) 0
	puts $fd [read $PGB(Logfd)]
	close $fd
	return 1
}

#
# GetDbs -	get a list of databases
#
# action:
#	try's env(USER) and then template1 databases to retrieve
#	list of all databases and updates PGB(Dbs) list if successful
#
proc GetDbs {} {
	global PGB env

        if ![CloseQuery] return
	set PGB(TopDb) {}
	set PGB(Db) {}
	set PGB(Dbs) {}
	set PGB(Classes) {}
	set PGB(Class) {}
	# get rid of any old InfoBox'es, clear root listboxes
	DelTopLevs
	foreach box {.dbs .classes .atts .types} {
		ClearBox $box.ls.l
		$box.t conf -text {}
	}
	.dbs.t conf -text $PGB(Host)

	# get list of dbs to try
	set topdbs template1
	if [info exists env(USER)] {set topdbs "$env(USER) $topdbs"}
	foreach d $topdbs {
		set PGB(Db) $d
		if [DoMonitor "retrieve(pg_database.datname)"] {
			set PGB(TopDb) $d
			set PGB(Db) {}
			break
		}
	}
	if {$PGB(TopDb) == ""} {
		ErrorBox "Could not get database list; Postgres error:\n\n$DMReturn"
		return 
	}

	set PGB(Dbs) [lsort $DMReturn]
	FillBox .dbs.ls.l $PGB(Dbs)
}

#
# GetClasses -	get a list of classes in the current database
#
# input:
#	d	name of database to query
# action:
#	fills PGB(Classes) with class list if successful
#
proc GetClasses { d } {
	if {$d == {}} return

	global PGB
	if { [lsearch $PGB(Dbs) $d] < 0 } {
		ErrorBox "No such database: $d"
		return
	}	
        if ![CloseQuery] return
	set PGB(Class) {}
	set PGB(Classes) {}
	set PGB(Db) $d
	# clear out all the messy attribute arrays maintained by Attribute
	Attribute reset
	# remove any InfoBox'es hanging around
	DelTopLevs
	foreach b {.classes .atts .types} {
		ClearBox $b.ls.l
		$b.t conf -text {}
	}
	.classes.t conf -text $PGB(Db)

	# skip postgres maintained stuff (pg_) and LO's (Xin)
	if ![DoMonitor {retrieve(pg_class.relname)
			where	pg_class.relname !~ "^pg_"
			and	pg_class.relname !~ "^Xin"}] {
		ErrorBox "Could not get classes list; Postgres error:\n\n$DMReturn"
		return 
	}
	set PGB(Classes) [lsort $DMReturn]
	FillBox .classes.ls.l $PGB(Classes)
}

#
# GetAttributes - get a list of attributes in the current class
#
# input:
#	c	class to query
#
# action
#	accesses and fills the arrays maintained by Attribute
#
proc GetAttributes { c } {
	if {$c == {}} return

	global PGB
	if { [lsearch $PGB(Classes) $c] < 0 } {
		ErrorBox "No such class: $c"
		return
	}	

        if {[CloseQuery] == 0} return
	set PGB(Class) $c

	DelTopLevs
	ClearBox .atts.ls.l
	ClearBox .types.ls.l
	.atts.t conf -text "$PGB(Class) names" 
	.types.t conf -text "$PGB(Class) types"

	if ![DoMonitor "retrieve(pg_attribute.attname,pg_type.typname)
			where pg_attribute.attrelid = pg_class.oid
			and   pg_attribute.atttypid = pg_type.oid
			and   pg_attribute.attnum   > 0
			and   pg_class.relname = \"$PGB(Class)\""] {
		ErrorBox "Could not get classes list; Postgres error:\n\n$DMReturn"
		return 
	}
	Attribute reset
	upvar #0 [Attribute typearray] Types
	upvar #0 [Attribute orderarray] Order
	# NB:
	# the Order array is the name of each attribute indexed
	# by the number of the column in which it appears in
	# the database.  This is the order in which they appeared
	# in the original Postquel CREATE command (pgbrowse's
	# create commands stuff things in alphabetically, other
	# users and/or tools might do things randomly).
	set n 0	
	foreach att $DMReturn {
		set name [lindex $att 0]
		set type [lindex $att 1]
		set Types($name) $type
		set Order($n) $name
		incr n 1
	}
	# NB: "Attibute names|types" returns stuff sorted by attribute name
	FillBox .atts.ls.l [Attribute names]
	FillBox .types.ls.l [Attribute types]
}

#
# ShowAttributeDesc - pop up an info box with a description of an attribute
#
# input:
#	att	attribute name in current class and db
#
# action:
#	querys the "class_info" class of textual description of the
#	attribute.  Almost nobody maintains this info so you'll likely
#	get nothing back.
#
proc ShowAttributeDesc { att } {
	global PGB

	if ![DoMonitor -return descrip "retrieve(class_info.attdescrip)
			where	class_info.class = \"$PGB(Class)\"
			and	class_info.attname = \"$att\""] {
		set descrip "No description available."
	}
	if {![info exists descrip]} { 
		set descrip "No description available."
	}

	InfoBox "Attribute Info\n\nHost:\t$PGB(Host)\nDatabase:\t$PGB(Db)\nClass:\t$PGB(Class)\n\n$att:\n\n$descrip\n"

}

#
# ShowTypeDesc - pop up an info box with a description of a type
#
# input:
#	name	name of type
#
# action:
#	uses TypeInfo to get info on the type
#
proc ShowTypeDesc { name } {
	global PGB

	if {[set desc [TypeInfo desc $name]] == {}} {
		set desc "No description available."
	}
		
	InfoBox "Type Info\n\nHost:\t$PGB(Host)\nDatabase:\t$PGB(Db)\nClass:\t$PGB(Class)\n\n$name:\n\n$desc"

}

#
# SetPGHost -	prompt for a new pghost
#
# action:
#	modifies PGB(Host) and calls GetDbs
#
proc SetPGHost { } {
	global PGB
	if {[set newhost [EntryDialog "Postgres Server" $PGB(Host) "Set Postgres Server"]] == {}} return
	set PGB(Host) $newhost
	set PGB(Db) {}
	GetDbs

}

#
# NewPGBrowser - start a new PGBROWSE 
#
# action
#	exec's a new pgbrowser which fires up on the same host/db/class
#
proc NewPGBrowser {} {
	global PGB
	if {$PGB(Db) != {}} {
		set db "-db $PGB(Db)"
	} else {
		set db ""
	}
	if {$PGB(Class) != {}} {
		set class "-class $PGB(Class)"
	} else {
		set class ""
	}
	eval exec $PGB(Home)/pgbrowse -host "$PGB(Host)" $db $class &
}

#
# CreateDB -	create a new database on the current PGB(Host)
#
# action:
#	prompts for new db name and exec's createdb
proc CreateDB {} {
	global PGB
	if {$PGB(Createdb) == ""} {
		ErrorBox {Could not find the "createdb" program in your PATH.}
		return
	}
	if {[set newdb [EntryDialog {New Database} {} {Create New Database}]] == {}} return

	AddToLog "$PGB(Createdb) -h $PGB(Host) $newdb"  "Shell Command"
	if [catch {exec $PGB(Createdb) -h $PGB(Host) $newdb} err] {
		set errmsg "createdb failed for new database $newdb:\n\n$err"
		ErrorBox $errmsg
	  	AddToLog "errmsg" FAILED!
		return
	}
	AddToLog "" SUCCESS
	GetDbs
}

#
# UsageExit -	exit with a usage message
#
proc UsageExit {} {
	puts stderr {Usage: pgbrowse [-logfile filename] [-db database] [-class class] [-host pghost] [-query] [-newclass] [-showlog]}
	destroy .
	exit 1
}

#
# IcopyGui - start the Icopy interface
#
proc IcopyGui {} {
    global PGB
    exec $PGB(Home)/Icopy.tcl  &
}

#
# DestroyClass - destroy the currently selected class
#
proc DestroyClass {} {
    global PGB
    if ![CheckFor Class] return
    if ![Confirm "Destroy class $PGB(Class) in database $PGB(Db)?" "Yes, destroy" "No, keep"] return
    if ![DoMonitor "destroy $PGB(Class)"] {
         ErrorBox "Destory class failed, Postgres error:\n$DMReturn"
         return
    }
    GetClasses $PGB(Db)
}

#
# FillClass -	read an ordinary file into a Postgres class with the
#		Postquel COPY command
#
proc FillClass { } {
	global PGB
	if {![CheckFor Db] || ![CheckFor Class]} return
        if ![DoMonitor  "retrieve(pg_attribute.attname)
		         where pg_attribute.attrelid = pg_class.oid
			 and   pg_attribute.attnum   > 0
			 and   pg_class.relname = \"$PGB(Class)\""] {
		Waiting
		ErrorBox "Could not get classes list; Postgres error:\n\n$DMReturn"
		return 
	}
        toplevel .fclass
        wm title .fclass "Fill Class $PGB(Class)"
        frame .fclass.mbs -relief sunken -bd 2
        set PGB(FillNAtts) [llength $DMReturn]
        for {set n 1} {$n <= $PGB(FillNAtts)} {incr n 1} {
            set attname [lindex $DMReturn [expr $n-1]]
            set PGB(Column$n) $attname
            menubutton .fclass.mbs.$n -text "$n: $attname" -menu .fclass.mbs.$n.m \
                  -relief raised -bd 2 -width 16
            menu .fclass.mbs.$n.m
            foreach a $DMReturn {
                  .fclass.mbs.$n.m add radiobutton -variable PGB(Column$n) -value $a -label $a \
                        -command "AdjustColumns $n"

            }
            pack append .fclass.mbs .fclass.mbs.$n {left padx 5m pady 5m}
        }
        frame .fclass.d
        label .fclass.d.l -text Delimeter:
        entry .fclass.d.e -relief sunken -bd 2 -width 1
        label .fclass.d.fl -text Filename:
        entry .fclass.d.f -relief sunken -bd 2
        label .fclass.d.dirl -text Directory:
        entry .fclass.d.dir -relief sunken -bd 2
        .fclass.d.dir insert 0 [pwd]
        .fclass.d.e insert 0 \\t
        pack append .fclass.d \
              .fclass.d.l left .fclass.d.e left \
              .fclass.d.dir right .fclass.d.dirl right .fclass.d.f right .fclass.d.fl right
        mkMenuBar .fclass.menu { } {
             File {
                      {{Input File} FillClassFile {Set input file for COPY command.}}
                      {Execute {destroy .fclass} {Executes COPY.}}
                      {Cancel FillClassCancel {Cancel COPY.}}
             }
        }
        pack append .fclass \
             .fclass.menu {top fillx} \
             [label .fclass.l -text {Column Order} -anchor w] {top fillx} \
             .fclass.mbs {pady 5m top} \
             .fclass.d {bottom pady 5m frame w}
        tkwait window .fclass
        if {$PGB(FillFile) == {}} return
        if ![DoMonitor "copy $PGB(Class) from \"[pwd]/$PGB(FillFile)\""] {
             ErrorBox "COPY Error: $DMReturn"
        }
        return
}

#
# FillClassCancel - cancel the fill class procedure
#
proc FillClassCancel { } {
       global PGB
       if ![Confirm "Cancel COPY?" "Yes, cancel" "No, return"] return
       set PGB(FillFile) ""
       destroy .fclass
}

#
# FillClassFile - specify text file to COPY in
#
proc FillClassFile { } {
       global PGB
       set PGB(FillFile) [FDOpenFilename {File to COPY From}]
       .fclass.d.f delete 0 end
       .fclass.d.f insert 0 $PGB(FillFile)
       .fclass.d.dir delete 0 end
       .fclass.d.dir insert 0 [pwd]
}

#
# AdjustColumns - swaps columns in the FillClass dialog when the user
#		   moves an attribute to a different column
#
proc AdjustColumns { ncol } {
       global PGB
       set cur [lindex [lindex [.fclass.mbs.$ncol conf -text] 4] 1]
       set new $PGB(Column$ncol)
       for {set n 1} {$n <= $PGB(FillNAtts)} {incr n 1} {
             if {$n == $ncol} continue
             if {$new == $PGB(Column$n)} break
       }
       .fclass.mbs.$ncol conf -text "$ncol: $new"
       .fclass.mbs.$n conf -text "$n: $cur"
       set PGB(Column$n) $cur
}

#
# Attribute -	current attribute info procedure
#
# inputs:
#	cmd	command to execute, type, typearray, etc.
#	var	arg for some commands
#
# returns:
#	requested attribute info
#
proc Attribute { cmd {var {}}} {
	global PGBATypes PGBAOrder 

	case $cmd in {
                type    {
                        if {$var == {}} {return [Attribute types]}
                        if ![info exists PGBATypes($var)] return {}
                        return $PGBATypes($var)
                        }
		typea*  {return PGBATypes}
		ordera* {return PGBAOrder}
		names	{return [lsort [array names PGBATypes]]}
		types	{
			foreach n [Attribute names] {lappend r $PGBATypes($n)}
			return $r
			}
		num*	{return [array size PGBATypes]}
		nameo*  {
			set num [Attribute num]
			for {set n 0} {$n < $num} {incr n 1} {
				lappend r $PGBAOrder($n)
			}
			return $r
			}
		typeo* {
			foreach n [Attribute nameorder] {lappend r $PGBATypes($n)}
			return $r
			}
		reset {
			catch {unset PGBATypes PGBAOrder}
		      }
		default {
			error "Attribute error: no such command $cmd"
			}
	}
}

#
# CheckFor -	check that certain globals are defined
#
# input:
#	what	what to check for, Db or Class
#
# returns:
#	1	global exists
#	0	global not yet defined
#
proc CheckFor { what } {
     global PGB
     case $what in {
        Db {set msg database}
        Class {set msg class}
        default {set msg $what}
     }
     if {$PGB($what) == {}} {
            NoticeBox "Please select a $msg first."
            return 0
     }
     return 1
}

#
# InitPGBRoot -	initializes the pgbrowse root window, creating and packing
#		the menus, info bar, and listboxes together
#
proc InitPGBRoot {} {
	global PGB


	set defpath ~postgres/bin:/usr/local/postgres/bin:/usr/postgres/bin
	if {[set PGB(Monitor) [SearchPath monitor $defpath ]] == ""} {
		error {Could not find "monitor" in your PATH}
		destroy .
		exit 1
	}
	# find createdb - this isn't critical so we quietly except ""
	set PGB(Createdb) [SearchPath createdb $defpath]

	mkInfoBar .info -bitmap @$PGB(Home)/bitmaps/redwood \
		-text {Sequoia 2000 Database Browser}

	mkMenuBar .menu {
		{ {About PGBROWSE} @PGB_About }
		{ {Version} {Version 2.1} }
		{ {Bugs/Missing Features} @PGB_Bugs }
	} {
		File {
			{{Set Postgres Server} SetPGHost {Change the Postgres server host.  The initial server is set to your PGHOST environment variable if it exists or defaults to heel.s2k.berkeley.edu} }
			{{Show Log} OpenLog {Shows log of monitor commands in which you can insert notes about your work.} }
			{{Save Log} SaveLog {Save current log of monitor commands and any notes you entered.} }
			{{Quit} QuitPGB {Exits PGBROWSE.} }
		}
	} {
		Edit {
			{{New Browser} NewPGBrowser {Start a new Postgres Browser.} }
			{{New Database} CreateDB {Create a new database on the current Postgres server.  You must have "createdb" privilege for this to work.} }
			{{New Class} CreateClass {Creates a new class in the current database.} }
			{{Destroy Class} DestroyClass {Destroys the currently selected class in the current database.} }
			{{Icopy GUI} IcopyGui {Icopy User Interface.} }
		} 
	} {
		Query {
			{{Create and Execute Query Tool} {pg_gui 0} {Creates a User Interface for the current database and class allowing you to retrieve and append data. This tool will ultimately have hooks to 3rd party packages such as S, IDL, AVS, etc.} }
		{{Save Query Tool} {pg_gui 1} {Creates a User Interface for the current database and class and allows the user to save it as a file} }
		}
	}

	if $PGB(Debug) {
		AddMenuButtons .menu {Debug {
			  {{Re-Source pg_action.tcl} {ReSource pg_action.tcl}}
			  {{Re-Source pg_browse.tcl} {ReSource pg_browse.tcl}}
			  {{Re-Source pg_create.tcl} {ReSource pg_create.tcl}}
			  {{Re-Source pg_guibuilder.tcl} {ReSource pg_guibuilder.tcl}}
			  {{Re-Source pg_query.tcl} {ReSource pg_query.tcl}}
			  {{Re-Source tkutils.tcl} {ReSource tkutils.tcl}}
			  {{Re-Source FD.tcl} {ReSource FD.tcl}}
			  }
			}
		.menu.2.m add command -label {Another query tool} \
			-command QueryClass
		.menu.help.m.2 add command -label {Another query tool} \
			-command {HelpBox {This is a query tool integrated with pg browse which may prove useful in the future.}}
		.menu.1.m add command -label {Fill Class} \
			-command FillClass
		.menu.help.m.1 add command -label {Fill Class} \
			-command {HelpBox {A half-written command to fill a tabular class from an ordinary file using the Postquel COPY command.  Column rearranging doesn't work, the field delimiter has be to a <tab> no matter what you say, and the file has to be local to the Postgres server (although the File browser will pop up on the machine PGBROWSE is running on).}}
	}


	mkListBox .dbs { GetClasses [selection get] }
	mkListBox .classes { GetAttributes [selection get] }
	mkListBox .atts { ShowAttributeDesc [selection get] }
	mkListBox .types { ShowTypeDesc [selection get] }

	proc AttTypesScroll index {
		.atts.ls.l yview $index; .types.ls.l yview $index
	}
	.atts.ls.s configure -command AttTypesScroll
	.types.ls.s configure -command AttTypesScroll
	
	set where "left padx 10 pady 10"
	pack append . \
		.menu { top fillx } \
		.info { top fillx } \
		.dbs $where \
		.classes $where \
		.atts $where \
		.types $where
}

#
# ReSource -	re-source a tcl procedures file in the $PGB(Home)/lib
#		directory
#
# input:
#	file	tcl file to source
#
proc ReSource { file } { 
	global PGB
	set f $PGB(Home)/lib/$file
	if [file exists $f] {
		puts stderr "re-reading $f..."
		uplevel #0 "source $f"
	}
}
