#
# CreateClass -	entry point into all the crazy create class stuff
#
# action:
#	prompts for a new class - if it doesn't already exists,
#	constructs .cclass window where all the types are listed and
#	allows the user to build up a CREATE command of known types
#	(scalar or array).  Waits for .cclass to exit and sends command
#	to Postgres backend.
#
proc CreateClass {} {
	global PGB
	if ![CheckFor Db] return
	if {[set PGB(NewClass) [EntryDialog {New Class} {} {Create New Class}]] == "" } return
	if {[string length $PGB(NewClass)] > 13} {
		NoticeBox "Invalid class name:  \"$PGB(NewClass)\".  Class names must be < 13 characters long."
		CreateClass
		return
	}
	if [regexp {^[^a-zA-Z]|\ } $PGB(NewClass)] {
		NoticeBox "Invalid class name:  \"$PGB(NewClass)\".  Class names must begin with a letter and can not contain spaces."
		CreateClass
		return
	}
	if {[lsearch $PGB(Classes) $PGB(NewClass)] != -1} {
		NoticeBox "Class \"$PGB(NewClass)\" already exists in database $PGB(Db)"
		CreateClass
		return
	}
	toplevel .cclass
	wm title .cclass "Create New Class $PGB(NewClass)"

	mkMenuBar .cclass.menu {
		{ {About Adding a Class} {Build up your new class, attribute by attribute, by double clicking on the desired attribute type in the first (leftmost) box.  When finished, select "Execute" from the "File" menu to send the "postquel" command.} }
		{ {Adding Attributes} {To add an attribute, double click on the desired type in the first (leftmost) box.  You will be prompted for the new attribute's name.} }
		{ {Deleting Attributes} {To delete an attribute, double click on its entry in the second (middle) box.} } 
		} {
		File	{
			{ Execute ExecuteCreateClass {Sends the Postquel CREATE command to Postgres.} }
			{ Cancel CancelCreateClass {Exits the "Create Class" dialog, discarding the new class.} }
			}
		}

	frame .cclass.boxes -bd 2 -relief raised
        mkListBox .cclass.boxes.types {
                .cclass.e.t delete 0 end
		.cclass.e.t insert 0 [lindex [split [lindex [selection get] 0] :] 0]
        } 40x10
	.cclass.boxes.types.t conf -text "Type Names"
        mkListBox .cclass.boxes.added DelAttribute 40x10
	.cclass.boxes.added.t conf -text "Postquel"
	pack append .cclass.boxes \
		.cclass.boxes.types left \
		.cclass.boxes.added left 
	pack append .cclass \
		.cclass.menu { top fillx } \
		.cclass.boxes {top fillx}
	frame .cclass.e
	label .cclass.e.1 -text "Create"
	frame .cclass.e.r
        set PGB(SelectedDim) scalar
	radiobutton .cclass.e.r.scalar -variable PGB(SelectedDim) -value scalar \
		-text "a scalar" -relief flat
	radiobutton .cclass.e.r.variable -variable PGB(SelectedDim) -value variable \
		-text "variable length array" -relief flat
	radiobutton .cclass.e.r.fixed -variable PGB(SelectedDim) -value fixed \
		-text "fixed length array of size" -relief flat
	entry .cclass.e.r.size -width 5 -relief sunken -bd 2
	pack append .cclass.e.r \
		.cclass.e.r.scalar {frame w top } \
		.cclass.e.r.variable {frame w top } \
		.cclass.e.r.fixed {frame w left } \
		.cclass.e.r.size {frame w left }
	label .cclass.e.2 -text {}
	entry .cclass.e.t -relief sunken -bd 2 -width 10
	label .cclass.e.3 -text "named"
	entry .cclass.e.e -relief sunken -bd 2 -width 10
	mkButton .cclass.e.b Add AddAttribute default
	bind .cclass.e.e <Return> {.cclass.e.b flash;.cclass.e.b invoke}
	pack append .cclass.e \
		.cclass.e.1 left \
		.cclass.e.r {left frame w} \
		.cclass.e.2 left \
		.cclass.e.t left \
		.cclass.e.3 left \
		.cclass.e.e {left frame w padx 5m} \
		.cclass.e.b {left frame e padx 5m}
	pack append .cclass \
		.cclass.e {bottom frame w padx 5m pady 5m}
	foreach t [lsort [TypeInfo names]] {
                lappend types "$t:  [TypeInfo desc $t]"
        }
	FillBox .cclass.boxes.types.ls.l $types
        SetTabbing .cclass.e.t .cclass.e.e .cclass.e.b
	tkwait window .cclass
}

#
# CancelCreateClass - cancel create class stuff
#
# action:
#	prompts for confirmation before destroying create class
#
proc CancelCreateClass {} {
	global PGB
	if {![NewClassInfo num] || ![Confirm "Exit now and abandon your new class?" "No, continue" "Yes, abandon"] } {
		NewClassInfo reset
		destroy .cclass
	}
}

#
# ExecuteCreateClass - sends CREATE command to Postgres
#
# action:
#	sends CREATE command, deals with error or callsGetClasses
#	and then GetAttribute for the newclass
#
proc ExecuteCreateClass {} {
	global PGB
	if ![NewClassInfo num] {
		NoticeBox "You haven't added any attributes yet."
		return
	}

	destroy .cclass
	set cmd [CreateClassCmd]
	if [DoMonitor "$cmd"] {
		GetClasses $PGB(Db)
		GetAttributes $PGB(NewClass)
	} else {
		ErrorBox "Create class failed for class $PGB(NewClass); Postgres error:\n\n$DMReturn"
	}
	NewClassInfo reset
}

#
# CreateClassCmd - create a line-by-line list Postgres CREATE command
#		   for the class curently being added
#
proc CreateClassCmd { } {
	global PGB
	set num [NewClassInfo num]
	if !$num {return {}}

	set first 1
	for {set n 1} {$n <= $num} {incr n 1} {
		set name [NewClassInfo name $n]
		set basetype [NewClassInfo type $name]
		set basedesc [TypeInfo desc $basetype]
		set dims [NewClassInfo dims $name]
		case $dims in {
			s*	{set type $basetype
				 set desc "scalar $basedesc"
				}
			v*	{set type $basetype\[\]
				 set desc "variable length array of $basedesc"
				}
			default {set type $basetype\[$dims\]
				 set desc "size $dims array of $basedesc"
				}
		}
		if $first {
			set msg "create $PGB(NewClass) (\n  ${name}=$type    /* $desc */"
			set first 0
		} else {
			set msg "$msg,\n  ${name}=$type    /* $desc */"
		}
	}
	return "$msg\n)"
}

#
# DelAttribute - deletes an attribute from the new class listbox
#
proc DelAttribute { } {
	global PGB
	set cur [selection get]
	if ![string match *=* $cur] return
	set name [string trim [lindex [split [lindex $cur 0] =] 0]]
	if ![Confirm "Delete attribute $name?" "Yes, delete" "No, keep"] {
		return
	}
	NewClassInfo del [NewClassInfo index $name]
        FillAddBox
}

#
# AddAttribute - adds an attribute of the type specified in the current
#		 type entry box
#
proc AddAttribute { } {
	global PGB
	if {[set typename [.cclass.e.t get]] == "" } {
		NoticeBox {Please select a type.}
		return
	}
	if {[TypeInfo desc $typename] == ""} {
		ErrorBox "No such type:  \"$typename\""
		return
	}
	if {[set name [string trim [.cclass.e.e get]]] == ""} {
		NoticeBox "Please provide a name for your attribute"
		return
	}
	if [NewClassInfo index $name] {
		NoticeBox "Attribute $name has already been defined."
		return
	}
	if [regexp {^[^a-zA-Z]|\ } $name]  {
		ErrorBox "Invalid Name: \"$name\".  Attribute names must begin with an alphabetic character and contain no white space."
		return
	}
	
	case $PGB(SelectedDim) in {
		s*	{ set dims scalar }
		v*	{ set dims varable }
		f*	{ set dims [.cclass.e.r.size get]
			if {$dims == {}} {NoticeBox {Please select a size.}; return}
			if ![regexp {[0-9]*} $dims] {ErrorBox {Invalid array size - must be numeric.}; return}
			}
	}
	NewClassInfo add $name $typename $dims
	.cclass.e.e delete 0 end
	FillAddBox
}

#
# FillAddBox -	updates the new class attribute list box
#
proc FillAddBox { } {
	global PGB
	ClearBox .cclass.boxes.added.ls.l
        FillBox .cclass.boxes.added.ls.l [split [CreateClassCmd] \n]
}

proc NewClassInfo {cmd args} {
	global PGBAttName PGBAttType PGBAttDims PGBNAtts PGB
	if ![info exists PGBNAtts] {set PGBNAtts 0}
	case $cmd in {
		add {
			set name [lindex $args 0]
			set type [lindex $args 1]
			set dims [lindex $args 2]
			set n [incr PGBNAtts 1]
			set PGBAttName($n) $name
			set PGBAttType($name) $type
			set PGBAttDims($name) $dims
		}
		num {
			return $PGBNAtts
		}
		name {
			set n [lindex $args 0]
			return $PGBAttName($n)
		}
		dims {
			set name [lindex $args 0]
			return $PGBAttDims($name)
		}
		type {
			set name [lindex $args 0]
			return $PGBAttType($name)
		}
		del {
			set n [lindex $args 0]
			incr PGBNAtts -1
			for {set i $n} {$i <= $PGBNAtts} {incr i 1} {
				set next [expr $i+1]
				set PGBAttName($i) $PGBAttName($next)
			}
		}
		index {
			set name [lindex $args 0]
			if ![info exists PGBAttName] {return 0}
			foreach i [array names PGBAttName] {
				if {$PGBAttName($i) == $name} {return $i}
			}
			return 0
		}
		reset {
			catch {
				unset PGBNAtts
				unset PGBAttType
				unset PGBAttName
				unset PGBAttDims
				unset PGB(NewClass)
			}
		}
	}
}
