#
# QueryClass -	Query the current Postgres class
#
# action:
#	sets up an entry box for each attribute so searches can
#	be qualified
#
proc QueryClass {} {
	global PGB
	if {![CheckFor Db] || ![CheckFor Class]} return

	toplevel .q
	wm title .q "Query Class $PGB(Class)"

	set PGB(QueryMatchRegExp) 1

	frame .q.b -bd 1 -relief sunken
	mkButton .q.b.look "Look Up" QueryLookUp
	mkButton .q.b.enter "Enter into Database" QueryEnter
	mkButton .q.b.clear "Clear Entries" QueryClear
	mkButton .q.b.delete "Delete" QueryDelete
	mkButton .q.b.retall "Retrieve ALL" QueryRetrieveAll
	set where "left padx 2m pady 2m"
	pack append .q.b \
		.q.b.look $where \
		.q.b.enter $where \
		.q.b.clear $where \
		.q.b.delete $where \
		.q.b.retall $where

	mkMenuBar .q.menu {} {
		File {
			{{Look Up} QueryLookUp}
			{{Enter into Database} QueryEnter}
			{{Clear Entries} QueryClear}
			{{Delete Instance(s)} QueryDelete {Does nothing for now}}
			{{Retrieve All Instances} QueryRetrieveAll}
			{Close CloseQuery {Closes the Query Dialog and Query Results window.}}
		     }
	}

	mkInfoBar .q.info -bitmap @$PGB(Home)/bitmaps/redwood \
		-text {Sequoia 2000 Query Tool}

	pack append .q \
		.q.menu {top fillx} \
		.q.info {top fillx} \
		.q.b {top pady 5m expand frame center}

	set natts [Attribute number]
	set names [Attribute names]
	set types [Attribute types]
	set tablist {}
	for  {set i 0} {$i < $natts} {incr i 1} {
		set name [lindex $names $i]
		set type [lindex $types $i]
		set desc [TypeInfo desc "$type"]
		frame .q.$i
		label .q.$i.l -text "$name, $type ($desc)" -anchor w
		entry .q.$i.e -width 40 -relief sunken
		pack append .q.$i \
			.q.$i.l {left frame w} \
			.q.$i.e {right frame e}
		pack append .q .q.$i {top fillx}
		set tablist ".q.$i.e $tablist"
	}
	eval "SetTabbing $tablist"
}

#
# QueryClear -	clear all the query entry boxes
#
proc QueryClear { } { 
	global PGB
	set natts [Attribute number]
	for {set n 0} {$n < $natts} {incr n 1} {.q.$n.e delete 0 end}
} 

#
# QueryEnter -	add an instance to the class being queried
#
proc QueryEnter { } { 
	global PGB

	set natts [Attribute number]
	set names [Attribute names]
	set first 1
	for {set i 0} {$i < $natts} {incr i 1} {
		set name [lindex $names $i]
		set data [.q.$i.e get]
                if {$data == {}} {
                       if ![Confirm "$name field null, continue?"] return
		       }
	        if $first {
		  set enter_query "append $PGB(Class) (\n\t$name=\"$data\""
		  set first 0
		} else {
                  set enter_query "$enter_query,\n\t$name=\"$data\""
		}
	}
        set enter_query "$enter_query\n)"
        if ![DoMonitor -flags -N -nolog "$enter_query"] {
	       ErrorBox "APPEND Failed, Postquel error:\n$DMReturn"
               return
	}
	ShowQueryResult $DMReturn
} 

#
# QueryLookUp -	perform a qualified search
#
# action:
#	constructs a Postquel command based on the current contents
#	of the entries boxes, runs DoMonitor and passes the monitor
#	temporary results filename to ShowQueryResults
#
proc QueryLookUp { } {
  global PGB
  set first 1
  set natts [Attribute number]
  set names [Attribute names]
  set lookup {}
  for {set i 0} {$i < $natts} {incr i 1} {
	set name [lindex $names $i]
	set data [.q.$i.e get]
	if {$data == {}} continue
	set match =
	if $PGB(QueryMatchRegExp) {
		case [Attribute type $name] in {
        	   {*text *char*} {set match ~}
		}
	}
	if $first {
		set lookup "retrieve($PGB(Class).all)\n\twhere $PGB(Class).$name $match \"$data\""
                set first 0
        } else {
                set lookup "$lookup\n\tand $PGB(Class).$name $match \"$data\""
        }
  }
  if {$lookup == {}} {
        NoticeBox "Please qualify your search."
        return 0
  }
  if ![DoMonitor -file -flags -N -nolog "$lookup"] {
        ErrorBox "Retrieve failed, Postgres error:\n$DMReturn"
        return 0
  }
	ShowQueryResults $DMReturn
}

#
# QueryRetrieveAll - retrieve all instances in a class
#
# action:
#	like QueryLookUp but doesn't narrow the search at all
#
proc QueryRetrieveAll { } {
        global PGB
        if ![DoMonitor -file -flags -N -nolog "retrieve($PGB(Class).all)"] {
                 ErrorBox "Retrieve all failed, Postgres error:\n$DMReturn."
		return
	}
	ShowQueryResults $DMReturn
	return
}

#
# CloseQuery -	closes the query and query results windows
#
proc CloseQuery {} {
	global PGB
	if ![winfo exists .q] {return 1}
	DelProgressWin .q
	destroy .q
	if [winfo exists .qr] {destroy .qr}
	if [info exists PGB(Query,TmpFiles)] {
		if [Confirm "Save the temporary monitor output files:  $PGB(Query,TmpFiles)" No Yes] {
			eval exec rm -f $PGB(Query,TmpFiles)
			unset PGB(Query,TmpFiles)
		}
	}
}

#
# ShowQueryResults - shows the results of a query
#
# input:
#	file	filename of temporary DoMonitor output file
#
# action:
#	makes a listbox for each attribute and fills it with the
#	query results.
#
proc ShowQueryResults {file} {
        global PGB ResultsCol
	if [info exists ResultsCol] {unset ResultsCol}
	if ![file readable $file] {
		ErrorBox "Can't read Postgres output file: $file."
		return
	}
	Working
	set fd [open $file]
	set ncol [Attribute num]
        if [winfo exists .qr] {destroy .qr}
	set nin 0
	while {![eof $fd]} {
	  set line [gets $fd]
	  if {[string range $line 0 0] == "-" || $line == {}} continue
	  set line [split $line |]
	  for {set n 0} {$n < $ncol} {incr n 1} {
	    lappend ResultsCol($n) [string trim [lindex $line [expr $n+1]]]
	  }
	  incr nin 1
	}
	close $fd
	lappend PGB(Query,TmpFiles) $file
	if {$nin == 0} {
	  NoticeBox {Nothing retrieved.}
	  return
	}

	set headers [Attribute nameorder]
	toplevel .qr
	wm title .qr "Query Results"
        wm minsize .qr 1 1
	mkMenuBar .qr.menu {} {
	  File {
	    {Close {destroy .qr} {Closes the Query Results box.}}
	  }
	}
	AddActionButton .qr.menu
	pack append .qr .qr.menu {top fillx}

	for {set c 0} {$c < $ncol} {incr c 1} {
	  mkListBox .qr.$c {}
	  .qr.$c.t conf -text [lindex $headers $c]
	  FillBox .qr.$c.ls.l $ResultsCol($c)
	  pack append .qr .qr.$c {expand left}
	  .qr.$c.ls.s conf -command QueryResultsScroll  
	}
	Waiting
}

#
# QueryResultsScroll - scroll query result listboxes together
#
# input:
#	index	index passed by a scrollbar
#
# action:
#	scrolls all listboxes together (note:  you can still scroll
#	a single listbox using the middle mouse button)
#
proc QueryResultsScroll { index } {
  set ncol [Attribute num]
  for {set c 0} {$c < $ncol} {incr c 1} {.qr.$c.ls.l yview $index}
}



proc QueryDelete {} {
	InfoBox "This function hasn't been written yet."
}
